#ifndef _MGACORE_H
#define _MGACORE_H

#include <linux/version.h>
#include <asm/io.h>
#include <linux/pci.h>

#if (LINUX_VERSION_CODE >= 0x020300)

#include <linux/spinlock.h>
#include <linux/i2c-old.h>

#else /* (LINUX_VERSION_CODE >= 0x020300) */

#include <asm/spinlock.h>
#include <linux/i2c.h>
#define wait_queue_head_t struct wait_queue *
/* FIXME */
#define init_waitqueue_head(_queue) _queue

#if (LINUX_VERSION_CODE < 0x020100)
#define MODULE_AUTHOR(_author)
#define MODULE_DESCRIPTION(_description)
#endif /* (LINUX_VERSION_CODE < 0x020100) */

#endif /* (LINUX_VERSION_CODE >= 0x020300) */

struct mga_dev_s;
typedef void (*mga_interrupt_handler_t)(struct mga_dev_s *, struct pt_regs *);

struct mga_dev_s
{
	/*************************************************************
	 * Core parameters                                           *
	 *                                                           *
	 *************************************************************/
	struct pci_dev *dev;
	u8* fb;                 /* framebuffer address */
        u8* ctrl;	        /* control aperture address */
	u32 memory_size;	/* framebuffer memory size */
	u32 options;            /* options detected (like HAS_TUNER, ...) */
	int in_use;		/* indicates that the core driver
				   is already used by a V4L module
				   (mgadvd, mga_vid, mgavideo, etc...)
				*/
	u32 malloc_limit;	/* used by "memory management" routines */
	spinlock_t lock;

	/*************************************************************
	 * Interrupt handlers specific parameters                    *
	 *                                                           *
	 *************************************************************/
	/* dynamic functions handling the different interrupt requests :
	 * the interrupt index is just associated with the array index, so
	 * not all slots will be used
	 */
	mga_interrupt_handler_t irq_handler[64];
	int irq_inuse;

	/*************************************************************
	 * Codec specific fields                                     *
	 *                                                           *
	 *************************************************************/
	struct mga_codec_s *codec;

	/*************************************************************
	 * I2C specific fields                                       *
	 *                                                           *
	 *************************************************************/
        struct i2c_bus i2c;


	/*************************************************************
	 * BES specific parameters                                   *
	 *                                                           *
	 *************************************************************/
	struct mga_bes_s *bes;

	/*************************************************************
	 * GRABBER specific parameters                               *
	 *                                                           *
	 *************************************************************/
	struct mga_grab_s *grabber;
};

#define MGAVIDEO_HAS_TUNER   1
#define MGAVIDEO_HAS_AUDIO   2
#define MGAVIDEO_HAS_DECODER 4
#define MGAVIDEO_HAS_MJPEG   8
#define MGAVIDEO_HAS_TVOUT   16

#define MGA_IRQ_VINVSYNC	1
#define MGA_IRQ_CMDCMPL		2
#define MGA_IRQ_BUFLVL		4
#define MGA_IRQ_DCMPEOI		8
#define MGA_IRQ_EXTERNAL	64

#define PALWTADD        0x3c00
#define X_DATAREG       0x3c0a
#define VSTATUS		0x3e30
#define VICLEAR         0x3e34
#define VIEN            0x3e38
#define STATUS		0x1e14
#define IEN             0x1e1c
#define CRTCEXTADD	0x1fde
#define CRTCEXTREG	0x1fdf
#define CACHEFLUSH	0x1fff

/* indexed registers */
#define	XMULCTRL	0x19
#define XMISCCTRL       0x1e
#define XGENIOCTRL      0x2a
#define XGENIODATA      0x2b
#define XKEYOPMODE      0x51
#define XCOLMSK0RED     0x52
#define XCOLMSK0GREEN   0x53
#define XCOLMSK0BLUE    0x54
#define XCOLKEY0RED     0x55
#define XCOLKEY0GREEN   0x56
#define XCOLKEY0BLUE    0x57
#define SDA             0x10
#define SCL             0x20

#define I2C_DELAY   	10
#ifndef	I2C_BUSID_MGA
#define I2C_BUSID_MGA 	100
#endif

/**
 *	mga_read -	read a register (32 bits)
 *	@mga:	mga device instance
 *	@reg:	register to read
 *	
 *	Read a 32 bits register of the MGA card.
 *	Returns the value of the register read.
 */
static __inline__ u32 mga_read(struct mga_dev_s* mga, u32 reg)
{
        return readl(mga->ctrl + reg );
}

/**
 *	mga_writeb -	write a register (8 bits)
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	data to write
 *	
 *	Write one (1) byte to a register in the MGA card.
 */
static __inline__ void mga_writeb(struct mga_dev_s* mga, u32 reg, u32 data)
{
        writeb( data, mga->ctrl + reg );
}

/**
 *	mga_writew -	write a register (16 bits)
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	data to write
 *	
 *	Write a word (16 bits).
 */
static __inline__ void mga_writew(struct mga_dev_s* mga, u32 reg, u32 data)
{
        writew( data, mga->ctrl + reg );
}

/**
 *	mga_write -	write a register (32 bits)
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	data to write
 *	
 *	Write a 32 bits value.
 */
static __inline__ void mga_write(struct mga_dev_s* mga, u32 reg, u32 data)
{
        writel( data, mga->ctrl + reg );
}

/**
 *	mga_indirect_readb -	read an indirect register (byte)
 *	@mga:	mga device instance
 *	@reg:	register to read
 *	
 *	Read a byte from an indirect register in the MGA card.
 *	Returns the byte read.
 */
static __inline__ u8 mga_indirect_readb(struct mga_dev_s *mga, u8 reg)
{
	writeb(reg, mga->ctrl + PALWTADD);
	return readb(mga->ctrl + X_DATAREG);
}

/**
 *	mga_indirect_writeb -	write a byte into an indirect register
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@data:	byte to write
 *	
 *	Write a byte into an indirect register in the MGA card.
 */
static __inline__ void mga_indirect_writeb(struct mga_dev_s *mga, u8 reg, u8 data)
{
	writeb(reg, mga->ctrl + PALWTADD);
	writeb(data, mga->ctrl + X_DATAREG);
}

/**
 *	mga_indirect_and_or -	apply a and/or operation on a register
 *	@mga:	mga device instance
 *	@reg:	register to write
 *	@and:	AND operand
 *	@or:	OR operand	
 *	
 *	This function reads a byte from an indexed register, applies both
 *	the OR and AND operation on the read data, and writes the result
 *	back into the register.
 */
static __inline__ void mga_indirect_and_or(struct mga_dev_s *mga, u8 reg, u8 and, u8 or)
{
	u8 data = mga_indirect_readb(mga, reg);
	data &= and;
	data |= or;
	mga_indirect_writeb(mga, reg, data);
}

/* see mgacore.c for a detailed documentation of the public API */
struct mga_dev_s * mga_get_card(void);
void   mga_release_card(struct mga_dev_s *mga);
void * mga_alloc_buffer(struct mga_dev_s *mga, unsigned long size);
void   mga_free_buffer (struct mga_dev_s *mga, unsigned long size);
int    mga_request_irq (struct mga_dev_s *mga);
int    mga_release_irq (struct mga_dev_s *mga);
int    mga_enable_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler);
int    mga_disable_irq(struct mga_dev_s *mga, int irq);


#endif /* _MGACORE_H */
