#define __NO_VERSION__
#include <linux/delay.h>
#include "mgacore.h"


/****************************************************************************
 * mga i2c bus interface
 ****************************************************************************/

static void mgai2c_attach(struct i2c_bus* bus, int id)
{
        struct mga_dev_s* mga = (struct mga_dev_s*)bus->data;
	int	i;

        printk("mgavideo: i2c device attach: %d\n", id);

        switch (id) {
        case I2C_DRIVERID_MSP3400:
                mga->options |= MGAVIDEO_HAS_AUDIO;
                break;

        case I2C_DRIVERID_TUNER:
                mga->options |= MGAVIDEO_HAS_TUNER;
                break;

        case I2C_DRIVERID_VIDEODECODER:
                mga->options |= MGAVIDEO_HAS_DECODER;
		for(i=0; i < bus->devcount; i++) {
			if (bus->devices[i]->addr == 0xD8)
				mga->options &= ~MGAVIDEO_HAS_TVOUT;
			if (bus->devices[i]->addr == 0xDA)
				mga->options |= MGAVIDEO_HAS_TVOUT;
		}
                break;
        }
}

static void mgai2c_detach(struct i2c_bus* bus, int id)
{
        struct mga_dev_s* mga = (struct mga_dev_s*)bus->data;

	printk("mgavideo: i2c device detach: %d\n", id);

        switch (id) {
        case I2C_DRIVERID_MSP3400:
                mga->options &= ~MGAVIDEO_HAS_AUDIO;
                break;

        case I2C_DRIVERID_TUNER:
                mga->options &= ~MGAVIDEO_HAS_TUNER;
                break;

        case I2C_DRIVERID_VIDEODECODER:
                mga->options &= ~MGAVIDEO_HAS_DECODER;
                break;
        }
}

static void mgai2c_setlines(struct i2c_bus* bus, int ctrl, int data)
{
        struct mga_dev_s* mga = (struct mga_dev_s*)bus->data;

        if (ctrl) {
		writeb(XGENIOCTRL, mga->ctrl + PALWTADD);
		writeb(readb(mga->ctrl + X_DATAREG) & ~SCL, mga->ctrl + X_DATAREG);
        } else {
		writeb(XGENIODATA, mga->ctrl + PALWTADD);
		writeb(readb(mga->ctrl + X_DATAREG) & ~SCL, mga->ctrl + X_DATAREG);
		writeb(XGENIOCTRL, mga->ctrl + PALWTADD);
		writeb(readb(mga->ctrl + X_DATAREG) | SCL, mga->ctrl + X_DATAREG);
        }

        if (data) {
		writeb(XGENIOCTRL, mga->ctrl + PALWTADD);
		writeb(readb(mga->ctrl + X_DATAREG) & ~SDA, mga->ctrl + X_DATAREG);
        } else {
		writeb(XGENIODATA, mga->ctrl + PALWTADD);
		writeb(readb(mga->ctrl + X_DATAREG) & ~SDA, mga->ctrl + X_DATAREG);
		writeb(XGENIOCTRL, mga->ctrl + PALWTADD);
		writeb(readb(mga->ctrl + X_DATAREG) | SDA, mga->ctrl + X_DATAREG);
        }

        udelay(I2C_DELAY);
}

static int mgai2c_getdataline(struct i2c_bus* bus)
{
        struct mga_dev_s* mga = (struct mga_dev_s*)bus->data;

        writeb(XGENIOCTRL, mga->ctrl + PALWTADD);
        writeb(readb( mga->ctrl + X_DATAREG ) & ~SDA, mga->ctrl + X_DATAREG);
        writeb(XGENIODATA, mga->ctrl + PALWTADD);
        return (readb(mga->ctrl + X_DATAREG) & SDA) ? 1:0;
}

/**
 *	mgai2c_init -	initializes the MGA i2c bus
 *	@mga:	mga instance
 *
 *	This function prepares the i2c driver backend for the MGA, and
 *	initializes the i2c bus. This will start a bus scan
 *	discovering the attached devices such as the TV tuner or MJPEG
 *	codec.
 */
void mgai2c_init(struct mga_dev_s *mga)
{
	/* setup the i2c */
	mga->i2c.id = I2C_BUSID_MGA;
	mga->i2c.data = mga;
	mga->i2c.bus_lock = SPIN_LOCK_UNLOCKED;
	mga->i2c.attach_inform = mgai2c_attach;
	mga->i2c.detach_inform = mgai2c_detach;
	mga->i2c.i2c_setlines = mgai2c_setlines;
	mga->i2c.i2c_getdataline = mgai2c_getdataline;
	mga->i2c.i2c_read = NULL;
	mga->i2c.i2c_write = NULL;
	i2c_register_bus(&mga->i2c);
}

