
/*
 *
 * Copyright (C) 1999, 2000  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

#define TOAST_NAV

#include <oms/oms.h>
#include <oms/plugin.h>

#include "demux.h"

static struct {
	pthread_t loop;

	int end_loop;
	buf_t *buf;
	int status;

	pthread_mutex_t mutex;
	pthread_cond_t playing;

	plugin_t *plugin;
} demux;


int demux_open ()
{
	return 0;
}


/**
 *
 */

void *demux_do (void *foo)
{
	buf_entry_t buf_entry;

	while (!demux.end_loop) {
		if (demux.status == STATUS_PAUSE) {
			demux_set_status (demux.plugin, STATUS_PAUSE);
			pthread_mutex_lock(&(demux.mutex));
			pthread_cond_wait (&(demux.playing), &(demux.mutex));
			pthread_mutex_unlock(&(demux.mutex));

			demux_set_status (demux.plugin, STATUS_PLAY);
		}

		if (!demuxBufGet (demux.buf, BUF_ANY, &buf_entry)) {
			PLUGIN_OPS_CODEC(demux.plugin, read, demux.buf, &buf_entry);
			free (buf_entry.mem);
		}
	}

        return NULL;

#if 0
	while (!demux.end_loop) {
		if (demux.status == STATUS_PAUSE)
			pthread_cond_wait (&demux.playing, &demux.mutex);
 if (dxr2_codec.status == DXR2_PLAYMODE_PAUSED) {
                        dxr2_set_playmode (DXR2_PLAYMODE_PAUSED);
                        pthread_mutex_lock(&(dxr2_codec.mutex));
                        pthread_cond_wait (&(dxr2_codec.playing), &(dxr2_codec.mutex));
                        pthread_mutex_unlock(&(dxr2_codec.mutex));

                        dxr2_set_playmode (DXR2_PLAYMODE_PLAY);
                        dxr2_codec.status=DXR2_PLAYMODE_PLAY;
                }

		if (!demuxBufGet (demux.buf, BUF_ANY, &buf_entry)) {
			switch (buf_entry.buf_id) {
			case BUF_VIDEO:
				CODEC_WRITE_VIDEO (buf_entry.mem, buf_entry.mem_len);
				break;
			case BUF_AUDIO: 
				CODEC_WRITE_AUDIO (buf_entry.mem, buf_entry.mem_len);
				break;
			case BUF_OTHER:
				CODEC_WRITE_OTHER (buf_entry.mem, buf_entry.mem_len);
				break;
			case BUF_SUBPIC: {
				CODEC_WRITE_SPU (buf_entry.mem, buf_entry.mem_len);
				break;
			}
#ifdef TOAST_NAV
			case BUF_NAV: {
//				int i;

				fprintf (stderr, "\n");

				fprintf (stderr, "PCI sub_stream_id:\t%02X\n", buf_entry.data[0]);
				fprintf (stderr, "nav_pack_lbn:\t\t%02X%02X%02X%02X\n",
					buf_entry.data[1],
					buf_entry.data[2],
					buf_entry.data[3],
					buf_entry.data[4]);
				fprintf (stderr, "vobu_category:\t\t%02X%02X%02X%02X\n",
					buf_entry.data[5],
					buf_entry.data[6],
					buf_entry.data[7],
					buf_entry.data[8]);
				fprintf (stderr, "user operation ctrl:\t%02X%02X%02X%02X\n",
					buf_entry.data[9],
					buf_entry.data[10],
					buf_entry.data[11],
					buf_entry.data[12]);
				fprintf (stderr, "start PTM of VOBU:\t%02X%02X%02X%02X\n",
					buf_entry.data[13],
					buf_entry.data[14],
					buf_entry.data[15],
					buf_entry.data[16]);
				fprintf (stderr, "end PTM of VOBU:\t%02X%02X%02X%02X\n",
					buf_entry.data[17],
					buf_entry.data[18],
					buf_entry.data[19],
					buf_entry.data[20]);
				fprintf (stderr, "end PTM of VOBU se:\t%02X%02X%02X%02X\n",
					buf_entry.data[21],
					buf_entry.data[22],
					buf_entry.data[23],
					buf_entry.data[24]);
				fprintf (stderr, "cell time (30fps):\t%02X%02X%02X%02X\n",
					buf_entry.data[25],
					buf_entry.data[26],
					buf_entry.data[27],
					buf_entry.data[28]);
				
				for (i=29; i<buf_entry.data_len; i++)
					fprintf (stderr, "%02X ", buf_entry.data[i]);
				fprintf (stderr, "\n");
				break;
			}
#endif TOAST_NAV
			default:
//				fprintf (stderr, "demux: unhandled stream type\n");
			}
			
			free (buf_entry.mem);

		} // else	// else wait for buffer to fill
	}	
#endif

	return NULL;
}


/**
 *
 */

int demux_init (plugin_t *plugin, buf_t *_buf, char *output_path)
{
	demux.buf = _buf;
	demux.plugin = plugin;
	
	demux.status = STATUS_STOP;
	pthread_mutex_init(&demux.mutex, NULL);
	pthread_cond_init (&demux.playing, NULL);

	return PLUGIN_OPS_CODEC(plugin, init, _buf, output_path);
}


/**
 *
 */

int demux_set_status (plugin_t *plugin, u_int status)
{
	if (status == demux.status)
		return -1;

	if ((status == STATUS_STOP) && (demux.status == STATUS_PAUSE))
		return -1;

	if ((status == STATUS_PAUSE) && (demux.status == STATUS_STOP))
		return -1;

	switch (status) {
	case STATUS_PLAY:
		if (demux.status == STATUS_PAUSE) {
			pthread_mutex_lock(&(demux.mutex));
			pthread_cond_signal (&demux.playing);
			pthread_mutex_unlock(&(demux.mutex));
		} else {
			demux.end_loop = 0;
			pthread_create (&demux.loop, NULL, demux_do, NULL);
#ifdef DEBUG
			fprintf (stderr, "THREAD CREATED demux.c:demux_set_status\n");
#endif
		}	
		PLUGIN_OPS_CODEC(plugin, set_status, STATUS_PLAY);
		break;
	case STATUS_PAUSE:
		PLUGIN_OPS_CODEC(plugin, set_status, STATUS_PAUSE);
		break;
	case STATUS_STOP:
		PLUGIN_OPS_CODEC(plugin, set_status, STATUS_STOP);
		demux.end_loop = 1;

		pthread_cancel (demux.loop);
		pthread_join (demux.loop, NULL);
/*
		if (demux.status == STATUS_PAUSE) {
			pthread_mutex_lock(&(demux.mutex));
			pthread_cond_signal (&(demux.playing));
			pthread_mutex_unlock(&(demux.mutex));
		}
*/

#ifdef DEBUG
		fprintf (stderr, "THREAD JOINED demux.c:demux_set_status\n");
#endif

		break;
	default:
		fprintf (stderr, "demux_set_status: unhandled status %d\n", status);
		return -1;
	}

	demux.status = status;

	return 0;
}
