
/*
 *
 * Copyright (C) 2000  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


//#define TOAST_URI

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>

#include <pthread.h>

#include <oms/oms.h>
#include <oms/plugin.h>
#include <oms/buf.h>

#include "demux.h"

#ifndef TOAST_URI 
#define DEVICE_NAME_LEN 8
#define DEVICE_VCD "/dev/vcd"
#define DEVICE_DVD "/dev/dvd"
#define DEVICE_DVB "/dev/dvb"
#define DEVICE_CDROM "/dev/cdrom"
#endif

typedef struct {
	char *root;		// rootpath or file

	struct {
		int audio;
		int subpic;

		int title;
		int chapter;

		int status;
	} current;

	pthread_mutex_t mutex;
	struct {
		pthread_mutex_t mutex;
		pthread_cond_t playing;
	} ctrl;

	struct {
		void (*chapter) (void *data, int chapter);
		void *chapter_data;
	} callb;        // Callbacks

	pthread_t play_loop;

	int fd;

	plugin_t plugin;

	buf_t buf;
} priv_t;

static priv_t foo;
priv_t *priv = &foo;


/**
 * open input device/file and associated plugins
 */

#ifdef TOAST_URI
int omsOpenInput (char *uri)
{
	struct stat info;
	char	*ptr,
		*uri_type,
		*uri_path;

fprintf (stderr, "OPEN INPUT\n");
// detect URI type && URI path description
//	file:/mnt/data/toast.ps
//	dvd:/mnt/dvd

	if ((ptr = strchr (uri, ':'))) {
		uri_type = uri;
		*ptr = '\0';
		uri = ptr+1;
	} else
		uri_type = '\0';

	uri_path = uri;

#else TOAST_URI
int omsOpenInput (char *uri_path)
{
	struct stat info;
#endif TOAST_URI
// just to be sure to have old stuff closed, before heading for new things 
	omsCloseInput ();

	if (stat (uri_path, &info) == -1) {
		omsPrint (OMS_PRINT_DEBUG, "path (%s) NOT found", uri_path);
		goto error;
	}

	omsPrint (OMS_PRINT_DEBUG, "path (%s) found", uri_path);

	priv->current.status = STATUS_STOP;
	priv->root = uri_path;

#ifdef TOAST_URI
	if (!stricmp (uri_type, "dvd")) {
		if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_DVD, NULL) < 0)
			goto error;
		return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
	} else if (!stricmp (uri_type, "dvb")) {
		omsPrint (OMS_PRINT_DEBUG, "found DVB device");
		goto error;
	} else if (!stricmp (uri_type, "vcd")) {
		if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_VCD, NULL) < 0)
			goto error;
		return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
	} else if (!stricmp (uri_type, "cdrom")) {
		if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_CDROM, NULL) < 0)
			goto error;
		return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
	} else if (!stricmp (uri_type, "file")) {
		if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_FILE, NULL) < 0)
			goto error;
		return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
	} else {
		omsPrint (OMS_PRINT_DEBUG, "UNKNOWN uri_type: %s", uri_type);
	}
#else
	if (S_ISCHR(info.st_mode) || S_ISBLK (info.st_mode)) { // device
		if (!strncmp (uri_path, DEVICE_DVD, DEVICE_NAME_LEN)) {
			if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_DVD, NULL) < 0)
				goto error;
			return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
		} else if (!strncmp (uri_path, DEVICE_VCD, DEVICE_NAME_LEN)) {
			if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_VCD, NULL) < 0)
				goto error;
			return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
		} else if (!strncmp (uri_path, DEVICE_DVB, DEVICE_NAME_LEN)) {
			omsPrint (OMS_PRINT_DEBUG, "found DVB device");
			goto error;
		} else if (!strncmp (uri_path, DEVICE_CDROM, DEVICE_NAME_LEN)) {
			if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_CDROM, NULL) < 0)
				goto error;
			return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
		}
	} else if (S_ISREG (info.st_mode)) {        // file
		if (pluginOpen (&priv->plugin, PLUGIN_ID_NAV, DL_NAV_FILE, NULL) < 0)
			goto error;
		return PLUGIN_OPS_NAV(&priv->plugin)->open (&priv->plugin, uri_path);
	} else {
		omsPrint (OMS_PRINT_DEBUG, "UNKNOWN");
	}
#endif

#if 0
	if (!plugin->codec) {
		omsPrint (OMS_PRINT_DEBUG, "no codec regstered!");
        } else
                plugin->decaps->write = plugin->codec->read;
#endif

error:
        return -1;
}


/**
 * close current input device/file and associated plugins
 */

int omsCloseInput (void)
{
#ifdef DEBUG
	fprintf (stderr, "CLOSE INPUT\n");
#endif

	// stop player if still running
	omsSetStatus (STATUS_STOP);

	// clean up plugins
	pluginClose (&priv->plugin, PLUGIN_ID_NAV);
	pluginClose (&priv->plugin, PLUGIN_ID_DECAPS);
	pluginClose (&priv->plugin, PLUGIN_ID_INPUT);

	return 0;
}


/**
 * open output device (and load relevant plugins)
 */

int omsOpenOutput (char *plugin_path, char *output_path)
{
	void *handle;
	plugin_codec_t *(*codec) (void);

	pthread_mutex_init(&priv->mutex, NULL);

	if (!(handle = _pluginOpen (&priv->plugin, PLUGIN_ID_CODEC, plugin_path)))
		return -1;

	if (!(codec = dlsym (handle, "plugin_init"))) {
		fprintf(stderr, "%s\n", dlerror());
		return -1;
	}

	_PLUGIN_OPS_CODEC(&priv->plugin) = (*codec) ();

	if (!PLUGIN_OPS_DECAPS(&priv->plugin))
		omsPrint (OMS_PRINT_DEBUG, "no decapsulation registered");
	else
		PLUGIN_OPS_DECAPS(&priv->plugin)->write = demuxBufPut;

	return demux_init (&priv->plugin, &priv->buf, output_path);
}


/**
 * clean all we've messed around during outputting). ok - frankly speaking,
 * this still needs a little bit of work.
 */

int omsCloseOutput (void)
{
	pluginClose (&priv->plugin, PLUGIN_ID_CODEC);
	return 0;
}


/**
 *
 */

nav_tree_t *omsGetInfo (u_int id)
{	nav_tree_t *tree = NULL;

	if (PLUGIN_OPS_NAV(&priv->plugin))
		tree = PLUGIN_OPS_NAV(&priv->plugin)->get_info (id);

	return tree;
}


/**
 * Main Loop.
 */

static void *omsLoop (void *foo)
{
	omsPrint (OMS_PRINT_DEBUG, "omsLoop: decoder loop started\n");

	demuxBufInit (&priv->buf);
	demux_set_status (&priv->plugin, STATUS_PLAY);

	while ((priv->current.status != STATUS_STOP) &&
	       (priv->current.status != STATUS_FINISH)) {

		if (priv->current.status == STATUS_PAUSE) 
			pthread_cond_wait (&priv->ctrl.playing, &priv->ctrl.mutex);
		if (PLUGIN_OPS_DECAPS(&priv->plugin)->decaps (&priv->buf))
			priv->current.status = STATUS_FINISH;
	}

	if (priv->callb.chapter)
		priv->callb.chapter (priv->callb.chapter_data, 0);

	pthread_exit (NULL);
}


/**
 * Register callback
 */

int omsRegisterCb (u_int id, void (*cb) (void *, int ), void *data)
{
	priv->callb.chapter = cb;
	priv->callb.chapter_data = data;

	if (PLUGIN_OPS_NAV(&priv->plugin)->register_callb) {
		PLUGIN_OPS_NAV (&priv->plugin)->register_callb (id, cb, data);
		return 0;
	}

	return -1;
}


/**
 * Unregister callback
 */

int omsUnregisterCb (u_int id)
{
	if (PLUGIN_OPS_NAV (&priv->plugin)->unregister_callb) {
		PLUGIN_OPS_NAV (&priv->plugin)->unregister_callb (id);
		return 0;
	}

	return -1;
}


/**
 * OMS set staus
 */

int omsSetStatus (u_int status)
{
	switch (status) {
		case STATUS_PAUSE_ON:
			if (priv->current.status != STATUS_PLAY)
				goto error;

			demux_set_status (&priv->plugin, STATUS_PAUSE);
			priv->current.status = STATUS_PAUSE;
			break;
		case STATUS_PAUSE_OFF:
			if (priv->current.status != STATUS_PAUSE)
				goto error;

			demux_set_status (&priv->plugin, STATUS_PLAY);
			priv->current.status = STATUS_PLAY;
			pthread_cond_signal (&priv->ctrl.playing);
			break;
		case STATUS_STOP:
			if (priv->current.status == STATUS_STOP)
				goto error;

			priv->current.status = STATUS_STOP;
			pthread_cancel (priv->play_loop);
			pthread_join (priv->play_loop, NULL);

			demux_set_status (&priv->plugin, STATUS_STOP);
			// close open descriptors here
			break;
		default:
			goto error;
	} 

	return 0;

error:
	return -1;
}


/**
 * get internal information of oms status
 */

int omsGetStatus (void)
{
	int ret;

	ret = priv->current.status;

	return ret;
}


/**
 * omsSetFilter - set stream filtering
 * @id: stream_id
 * @val: values to set filter to
 */

int omsSetFilter (u_int id, int val)
{
	switch (id) {
	case BUF_AUDIO: {
		plugin_codec_attr_audio_t attr = {-1, -1, -1, val, -1};

		priv->current.audio = val;

		if (PLUGIN_OPS_DECAPS(&priv->plugin)->set_filter (BUF_AUDIO, val))
			goto error;

		if (PLUGIN_OPS_CODEC(&priv->plugin, set_attributes, BUF_AUDIO, &attr))
			goto error;
		break;
	}
	case BUF_SUBPIC: {
		plugin_codec_attr_spu_t spu;
		spu.clut = NULL;
		spu.num = val;

		if (PLUGIN_OPS_DECAPS(&priv->plugin)->set_filter (BUF_SUBPIC, val))
			goto error;

		if (PLUGIN_OPS_CODEC(&priv->plugin, set_attributes, BUF_SUBPIC, &spu))
			goto error;

		priv->current.subpic = val;
	}
	}

	return 0;

error:
	return -1;
}


/**
 * omsGetFilter - get actual filter settings
 * @id: stream_id
 */

int omsGetFilter (u_int id)
{
	switch (id) {
	case BUF_AUDIO:
		return priv->current.audio;
	case BUF_SUBPIC:
		return priv->current.subpic;	
	}

	return -1;
}


//////////////////////////////////////////////////////////////////
/**
 * Play the specified title of the DVD
 * Title:  1..99
 */

int TitlePlay (int title)
{
#ifdef DEBUG
	fprintf(stderr, "oms' PartPlay sending track %d\n", title);
#endif
	return PartPlayAutoStop (title, 0, 0);
} 


/**
 * Play the specified title of the DVD, from the specified chapter on
 * Title:  1..99
 * Part:   1..999
 */

int PartPlay (int title, int chapter)
{
	return PartPlayAutoStop (title, chapter, 0);
}


/**
 *
 */

int PartPlayAutoStop (int title, int chapter, int parts2play)
{
        int ret = PLUGIN_OPS_NAV (&priv->plugin)->play (title, chapter, parts2play);

        if (priv->current.status == STATUS_STOP) {
                pthread_mutex_init(&priv->ctrl.mutex, NULL);
                pthread_cond_init (&priv->ctrl.playing, NULL);

                priv->current.status = STATUS_PLAY;

                pthread_create (&priv->play_loop, NULL, omsLoop, NULL);
        } else {
                priv->current.status = STATUS_PLAY;
                pthread_cond_signal (&priv->ctrl.playing);
        }

	return ret;
}

/**
 *
 */

int GetRoot (char *Root, int BufSize, int *ActualSize)
{
	strncpy (Root, priv->root, BufSize);
	*ActualSize=strlen (priv->root);

	return 0;
}


/**
 *
 */

int TimeSearch (u_long TimeCode)
{
	int hour,min,sec,frame,rate;
	TimeCode2Time(TimeCode,hour,min,sec,frame,rate);

	return 0;
}


/**
 *
 */

int GetTotalTitleTime (u_long *TotalTime)
{
	int tot=0;		//=_player_total_time();
	if (tot == -1)
		return -1;

	*TotalTime = Time2TimeCode(tot/3600,tot%3600/60,tot%60,0,0);
	return 0;
}



/**
 *
 */

int GetCurrentLocation (int *TitleNum, int *PartNum, u_long *TimeCode)
{
	int time=0;	//=_player_current_time();

	if (time == -1)
		return -1;

	*TitleNum = priv->current.title;
	*PartNum  = priv->current.chapter;
	*TimeCode = Time2TimeCode(time/3600,time%3600/60,time%60,0,0);

	return 0;
}


/**
 *
 */

int VideoPresentationModeChange (int InitialDisplay, int CurrentDisplay)
{
	plugin_codec_attr_video_t attr = {-1, -1, CurrentDisplay, InitialDisplay, -1};

	if (InitialDisplay != 0 && InitialDisplay != 3 &&
		CurrentDisplay >= 0 && CurrentDisplay <= 2)
		return -1;

	if (PLUGIN_OPS_CODEC(&priv->plugin, set_attributes, BUF_VIDEO, &attr))
		return -1;

	return 0;
}


/**
 *
 */

int PartSearch (int Part)
{
	if (Part < 1 || Part > 999)
		return -1;

	return 0;
}


