
/*
 *
 * Copyright (C) 2000  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */

#include <stdio.h>
#include <dlfcn.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>

#include <config.h>
#include <oms/oms.h>
#include <oms/plugin.h>

/**
 * Open and initialize plugin
 **/

int pluginOpen (plugin_t *plugin, u_int plugin_id, char *plugin_name, void *data)
{
	void *(*init)(void *);

	omsPrint (OMS_PRINT_DEBUG, "try to open plugin %s", plugin_name);

	if (plugin_id >= MAX_PLUGIN_ID) {
		fprintf (stderr, "unknown plugin type\n");
		return -1;
	}

	if (!(plugin->handle[plugin_id] = dlopen (plugin_name, RTLD_LAZY))) {
		fprintf (stderr, "PLUGIN: %s doesn't seem to be installed (%s)\n", plugin_name, dlerror ());

		return -1;
	}

	init = dlsym (plugin->handle[plugin_id], "plugin_init");

//DENT a hack for now:
	if (!data)
		data = plugin_name;

	if (!(plugin->ops[plugin_id] = (*init) (data))) {
//	if (!(plugin->ops[plugin_id] = (*init) (plugin_name))) {
		fprintf (stderr, "%s: %s unable to initialize (%s)\n", __FILE__, plugin_name, dlerror ());
		return -1;
	}

	omsPrint (OMS_PRINT_DEBUG, "plugin %s opened and initialized", plugin_name);

	return 0;
}


/**
 * Open plugin
 **/

void *_pluginOpen (plugin_t *plugin, u_int plugin_id, char *plugin_name)
{
	if (plugin_id >= MAX_PLUGIN_ID) {
		fprintf (stderr, "unknown plugin type\n");
		return NULL;
	}

	if (!(plugin->handle[plugin_id] = dlopen (plugin_name, RTLD_LAZY))) {
		fprintf (stderr, "PLUGIN: %s:%s\n", plugin_name, dlerror ());

		return NULL;
	}

	return plugin->handle[plugin_id];
}


/**
 * Close plugin and clean up plugin structure 
 **/

void *pluginClose (plugin_t *plugin, u_int plugin_id)
{
	if (plugin_id >= MAX_PLUGIN_ID) {
		fprintf (stderr, "unknown plugin type\n");
		return NULL;
	}

//DENT call plugin->*->close() here ...
	if (plugin->handle[plugin_id])
		dlclose (plugin->handle[plugin_id]);
	plugin->ops[plugin_id] = NULL;

	return NULL;
}


/**
 *
 **/

// DENT: should work as follows (in the future)
//	1) compare fileextension
//		if extension matches, check if magic also matches
//	2) if no filename matches, check all extensions
//
//	OR ... just use it like now - just check magic codes
//	i think i'll implment it like now, but save all magic codes into a list
//	so i don't have to dlopen the plugin first.
 
static int _pluginGetDecapsMagic (plugin_t *plugin, char *plugin_name, u_char *buf)
{
	plugin_decaps_magic_t *magic;
	int i;

	pluginOpen (plugin, PLUGIN_ID_DECAPS, plugin_name, NULL);
	magic = PLUGIN_OPS_DECAPS(plugin)->magic ();

	if (magic) {
#ifdef DEBUG
		fprintf (stderr, "%s: file: %s\n", plugin_name, magic->file_ext);
		fprintf (stderr, "%s: len: %d\n", plugin_name, magic->magic_len);

		fprintf (stderr, "%s: magic: ", plugin_name);
		for (i=0; i<magic->magic_len; i++)
			fprintf (stderr, "0x%02x ", magic->magic[i]);
		fprintf (stderr, "\n\n");
#endif

		if (!(strncmp (buf, magic->magic, magic->magic_len))) {
			fprintf (stderr, "%s matches -> found the right decapsulation for out file\n", plugin_name);
			return 0;
		}

	} else
		fprintf (stderr, "%s: NO MAGIC!\n\n", plugin_name);

	pluginClose (plugin, PLUGIN_ID_DECAPS);

	return -1;
}


/**
 * DENT: i know this is an ugly contruct right now, but:
 *	1) if we've an install routine, plugins will be looked up
 *		in the directory
 **/

int pluginGetDecapsMagic (plugin_t *plugin, char *filename)
{
	buf_entry_t buf_entry;
	int i;

	buf_entry.mem = malloc (20);
	buf_entry.mem_len = 20;

	PLUGIN_OPS_INPUT(plugin)->seek (0);
	PLUGIN_OPS_INPUT(plugin)->read (NULL, &buf_entry);

//DENT: just for now to list plugins - will be part of autodetection
	pluginScan (OMS_PLUGINDIR);

#ifdef DEBUG
	for (i=0; i<20; i++)
		fprintf (stderr, "0x%02x ", buf_entry.mem[i]);
#endif

	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_PS, buf_entry.mem))
		goto found;
	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_TS, buf_entry.mem))
		goto found;
	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_AVP, buf_entry.mem))
		goto found;
	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_VCD, buf_entry.mem))
		goto found;
// fallback
	pluginOpen (plugin, PLUGIN_ID_DECAPS, DL_DECAPS_DUMMY, NULL); 

found:	
	free (buf_entry.mem);
	return 0;
} 
