//PLUGIN_INFO(INFO_NAME, "LPCM decoder");
//PLUGIN_INFO(INFO_AUTHOR, "Thomas Mirlacher <dent@cosy.sbg.ac.at>");

#include <inttypes.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <oms/oms.h>
#include <oms/plugin/codec_audio.h>

static int is_output_initialized = 0;

static int _lpcmdec_open (plugin_t *plugin, void *foo);
static int _lpcmdec_close (plugin_t *plugin);
static int _lpcmdec_read (plugin_codec_audio_t *plugin, buf_t *buf, buf_entry_t *buf_entry);

static plugin_codec_audio_t codec_lpcmdec = {
	open:		_lpcmdec_open,
	close:		_lpcmdec_close,
	read:		_lpcmdec_read,
};


int fd;

/**
 *
 **/

static int _lpcmdec_open (plugin_t *plugin, void *foo)
{
	codec_lpcmdec.output = (plugin_output_audio_t *) foo;
	fd = open ("/tmp/sound.out", O_WRONLY);
	return 0;
}


/**
 *
 **/

static int _lpcmdec_close (plugin_t *plugin)
{
	close (fd);
	return 0;
}


/**
 *
 **/

static int _lpcmdec_read (plugin_codec_audio_t *plugin, buf_t *buf, buf_entry_t *buf_entry)
{
	int i;
	uint8_t *ptr = buf_entry->data;
	uint8_t obuf[4096];
	uint32_t bytes_left;

	if(!is_output_initialized) {
		codec_lpcmdec.output->setup (16, 24000, 2);
		is_output_initialized = 1;
	}

// FIXME: this is for downsampling (should be done by the output layer

	for (i=0; i<buf_entry->data_len; i+=8) {
		obuf[i/2+0] = ptr[i+0];
		obuf[i/2+1] = ptr[i+2];
		obuf[i/2+2] = ptr[i+4];
		obuf[i/2+3] = ptr[i+6];
	}

	bytes_left = buf_entry->data_len/4;

	codec_lpcmdec.output->write ((void *) obuf, buf_entry->data_len/2);

	return 0;	
}


/*****************************************/


/**
 * Initialize Plugin.
 **/


void *plugin_init (char *whoami)
{
	pluginRegister (whoami,
		PLUGIN_ID_CODEC_AUDIO,
		"lpcm",
		&codec_lpcmdec);

	return &codec_lpcmdec;
}


/**
 * Cleanup Plugin.
 **/

void plugin_exit (void)
{
}

