/*
 * extract_subpic.c - extracts DVD subtitles from a VOB file
 * Copyright (C) 2000   Samuel Hocevar <sam@via.ecp.fr>
 *	minor cleanups by <dent@cosy.sbg.ac.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *                                                     
 */


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>

int main (int argc, char *argv[])
{
        u_char buf[2048];
	u_int id;

        if (argc != 2) {
		fprintf(stderr, "usage: %s ID\nID : subtitle ID stream (0, 1, 2...)\nstdin : VOB(PS) stream\nstdout : extracted subtitles\n", argv[0]);
	
		exit (-1);
	}

	id = atoi(argv[1]) + 0x20;

	/* read the stream: 2048 bytes each time */
	while (read (0, &buf, 2048) == 2048) {
		if ((buf[0] != 0x00) || (buf[1] != 0x00) || (buf[2] != 0x01) || (buf[3] != 0xba)) {
			fprintf (stderr,"Sorry, doesn't appear to be a valid VOB(PS) stream\n");
			exit (0);
		}
			    
		if ((buf[14] == 0x00) && (buf[15] == 0x00) && (buf[16] == 0x01) && (buf[17] == 0xbd)) {
			u_int type = buf [23 + buf[22]];
			u_int packet_length = buf[18]<<8 | buf[19];

			if (type == id) { /* 0x2* is subtitle */
				write (1, buf + (24 + buf[22]), packet_length - buf[22] - 4);
				fprintf (stderr, "+");
			} else if ((type & 0xf0) == 0x20)
				fprintf (stderr, "-");
		}
	}
	fprintf(stderr, "\n");

	return 0;
}

