
/*
 * DUMMY DECAPSULATION
 *
 * Copyright (C) 2000  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>

#include <oms/plugin/decaps.h>

static int _dummy_open (plugin_decaps_t *plugin);
static int _dummy_close (plugin_decaps_t *plugin);
static int _dummy_decaps				(buf_t *buf);
static plugin_decaps_magic_t *_dummy_magic	(void);
static int _dummy_set_filter			(u_int filter_id, int val);

static plugin_decaps_t decaps = {
	_dummy_open,
	_dummy_close,
	_dummy_decaps,	// decasulation
	_dummy_magic,	// autodetect decapsulation
	_dummy_set_filter,// filtering
	NULL,		// read (inserted)
	NULL,		// write (inserted)
	2048,		// blocksize (inserted: this is a default)
	NULL		// private data
};

static buf_entry_t buf_entry; 
static buf_t *buf; 


static plugin_decaps_magic_t dummy_magic_entry = {
        "mp3",
        0,
        NULL
};


/**
 *
 **/

static int _dummy_open (plugin_decaps_t *plugin)
{
	buf_entry.mem_len = decaps.blocksize;

	return 0;
}


/**
 *
 **/

static int _dummy_close (plugin_decaps_t *plugin)
{
	return 0;
}


/**
 *
 */

static inline char *_get_bytes_block (void)
{
	if (!buf_entry.mem) {
		buf_entry.mem = valloc (decaps.blocksize);// check return val
		buf_entry.mem_len = decaps.blocksize; 
	}
	
	if ((buf_entry.mem_len = decaps.read (buf, &buf_entry)) <= 0)
		return NULL;

	buf_entry.data = buf_entry.mem;
	buf_entry.data_len = buf_entry.mem_len;
	buf_entry.type = BUF_TYPE_MP3;

	return buf_entry.mem;
}


/**
 *
 */

static int _dummy_decaps (buf_t *_buf)
{
	buf = _buf;

//get new block
	if (!(_get_bytes_block ()))
		return -1;

	buf_entry.flags = BUF_FLAG_NONE;
	buf_entry.buf_id = BUF_AUDIO;

	decaps.write (buf, &buf_entry);
	buf_entry.mem = NULL;

	return 0;
}


/**
 * Autodetect decapsulation method
 */

static plugin_decaps_magic_t *_dummy_magic (void)
{
	return &dummy_magic_entry;
}


/**
 * Filter streams
 */

static int _dummy_set_filter (u_int filter_id, int val)
{
	fprintf (stderr, "DUMY: no filter available\n");

	return 0;
}


/**
 * Initial condition for this nice little thingy
 */

void *plugin_init (char *whoami)
{
        pluginRegister (whoami,
                PLUGIN_ID_DECAPS,
                STREAM_ID_MPEG2,
                &decaps);

//FIXME
	_dummy_open (&decaps);
	
	return &decaps;
}


/**
 * Drop it
 */

void plugin_exit (void)
{
}

