#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "linux_raw.h"

#ifndef HAVE_LINUX_RAW_H
int raw_open (const char *dev) {
	return -1;
}
int raw_close (int fd) {
	return -1;
}
#else

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include <linux/raw.h>

static struct rawdev_name {
	char const *raw_ctl;
	char const *raw_nr;
} rawdev_names[] = {
	{ "/dev/raw",    "/dev/raw%d" },     /* devices.txt from >2.3.99 kernels */
	{ "/dev/rawctl", "/dev/raw/raw%d" }, /* found on redhat 6.2 */
	{ NULL }
};


/**
 *
 **/

int raw_open (const char *dev)
{
	struct stat info;
	struct rawdev_name *rd_name;
	int fd_rawctl, fd_raw;
	struct raw_config_request raw_conf;
	char devname[256];
   
/* Step 1: find raw control device */
	for(rd_name=rawdev_names;rd_name->raw_ctl!=NULL;++rd_name) {
#ifdef DEBUG
		fprintf(stderr,"RAW: opening raw control `%s'\n",rd_name->raw_ctl);
#endif      
		if ((fd_rawctl = open (rd_name->raw_ctl,O_RDWR))!=-1)
/* control device found */
			break;
/* control device not found */
		if(errno==ENOENT || errno==EISDIR)
			continue;

		fprintf(stderr,"while opening raw control device `%s': %s\n",rd_name->raw_ctl,strerror(errno));
		return -1;
	}

	if (!rd_name->raw_ctl) {
		fprintf (stderr,"no raw control device found\n");
		return -1;
	}

/* Step 2: examine device to open */
	if(stat(dev,&info)==-1) {
		fprintf(stderr,"stat(`%s'): %s\n",dev,strerror(errno));
		return -1;
	}

	raw_conf.block_major = major (info.st_rdev);
	raw_conf.block_minor = minor (info.st_rdev);

	for(raw_conf.raw_minor=1; raw_conf.raw_minor <= LINUX_RAWDEV_MAX; raw_conf.raw_minor++) {
#ifdef DEBUG
		fprintf(stderr,"RAW: binding to device %d\n",raw_conf.raw_minor);
#endif      
		if (ioctl (fd_rawctl, RAW_SETBIND, &raw_conf)) {
			if(errno==EBUSY)
				continue;
			fprintf(stderr,"ioctl(`%s',RAW_SETBIND,...): %s\n",rd_name->raw_ctl,strerror(errno));
			if(close (fd_rawctl))
				fprintf(stderr,"close(`%s'): %s\n",rd_name->raw_ctl,strerror(errno));
			return -1;
		}
		break;
	}
   
	if (close (fd_rawctl))
		fprintf(stderr,"close(`%s'): %s\n",rd_name->raw_ctl,strerror(errno));
   
	if (raw_conf.raw_minor==LINUX_RAWDEV_MAX) {
		fprintf(stderr,"no free raw device found\n");
		return -1;
	}

	snprintf(devname,sizeof(devname),rd_name->raw_nr,raw_conf.raw_minor);
#ifdef DEBUG
	fprintf(stderr,"RAW: opening device `%s'\n",devname);
#endif   
	if((fd_raw=open(devname,O_RDONLY))==-1) {
		fprintf(stderr,"open(`%s',O_RDONLY): %s\n",devname,strerror(errno));
		return -1;
	}
#ifdef DEBUG
	fprintf(stderr,"RAW: everything went ok\n");
#endif   
	return fd_raw;
}


/**
 *
 **/

int raw_close (int raw_fd)
{
	return close(raw_fd);
}
#endif

