#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>

#include <oms/oms.h>
#include <oms/buf.h>
#include <oms/plugin.h>
#include <oms/plugin/nav.h>
#include "udf/dvd_udf.h"
#include "ifo/libifo/ifo.h"

#define DVDDB

static int _dvd_open (plugin_t *plugin, char *path);
static int _dvd_PartPlayAutoStop (int title, int chapter, int parts2play);
static int _dvd_RegisterCb (u_int id, void (*cb) (void *, int ), void *data);
static int _dvd_UnregisterCb (u_int id);
static nav_tree_t *_dvd_get_info (u_int id);

static int _dvd_ifo_init ();
static int _dvd_next_cell_index (void);
static int _dvd_set_first_block ();
static int _dvd_next_block (char *data, u_int len);

#ifdef DVDDB
#include <dvddb/dvddb.h>
#endif

#define FILE_IFO "/VIDEO_TS/VIDEO_TS.IFO"

#define MAX_NEQ_PGS 10

static struct {
	int audio;
	int subpic;

	int title;
	int chapter;
        int angle;
        int num_angles;
        u_int num_cells;

        int seq_pgc, rnd_pgc;
        int num_pgc;

	long block;
	u_int title_start;
  
        u_int cell_index; // index to titel set cell address
        u_int next_cell_index;

        u_int cell_piece_index;
        //u_int cell_vob_id;
        //u_int cell_cell_id;

        u_int num_pgci_cells;
        u_int cur_pgci_cell_index;  // index to cell_play_info list
	u_int next_pgci_cell_index;
        u_int cells_per_angle;

        long last_angle_block;
        ifo_cell_addr_t *ptr_cell_addr; // start of title set cell addresses
        ifo_pgci_cell_addr_t *ptr_pgci_cell_addr; // start of pgci cell play info address 

        // non equivalent interleave stuff
        u_int non_equ_ilv;
        u_int ilv_pg_index;
        u_int num_ilv_pgms;
        u_int jump_cell[MAX_NEQ_PGS];
        u_int num_ilv_cell[MAX_NEQ_PGS];

	int fd_udf;

	ifo_t *ifo;
	char *root;

	plugin_t *plugin;

	struct {
		void (*chapter) (void *data, int chapter);
		void *chapter_data;
	} callb;        // Callbacks
} dvd_priv;


static plugin_nav_t dvd_nav = {
	_dvd_open,
	NULL,
	_dvd_PartPlayAutoStop,
	_dvd_get_info,
	_dvd_RegisterCb,	// register callback
	_dvd_UnregisterCb,	// unregister callback
	&dvd_priv
};


/**
 *
 */

static int _dvd_open (plugin_t *plugin, char *path)
{
	dvd_priv.root = path;
	dvd_priv.plugin = plugin;
	dvd_priv.angle = 0; // hard coded first angle
	dvd_priv.non_equ_ilv = 0;

	if (pluginOpen (plugin, PLUGIN_ID_INPUT, DL_INPUT_UDF, NULL))
		return -1;

	if (pluginOpen (plugin, PLUGIN_ID_DECAPS, DL_DECAPS_PS, NULL))
		return -1;

	PLUGIN_OPS_DECAPS(plugin)->blocksize = 2048;
	PLUGIN_OPS_INPUT(plugin)->blocksize  = 2048; 
	PLUGIN_OPS_INPUT(plugin)->open (path);
	PLUGIN_OPS_DECAPS(plugin)->read = _dvd_next_block;

	if (!plugin->ops[PLUGIN_ID_CODEC]) {
		omsPrint (LOG_DEBUG, "no codec registered");
		return -1;
	} else 
		PLUGIN_OPS_DECAPS(plugin)->write = demuxBufPut;

//DENT: check if we've a disc or a directory
	if ((dvd_priv.fd_udf = UDFOpenDisc (path)) < 0) {
		omsPrint (LOG_DEBUG, "unable to open IFO %s", path);
		return -1;
	}

	if (dvd_priv.ifo) {
		ifoClose (dvd_priv.ifo);
		close (dvd_priv.fd_udf);
	}

	if(_dvd_ifo_init () < 0) {;
                omsPrint (LOG_ERROR, "IFO initialization failed!");
                return -1;
        }                                                                          

// we need to close and reopen, cause we request access for a certain area
// of the disk each time we open - do a seek first to position to filepointer
//	to the right offset ...
	
	PLUGIN_OPS_INPUT(plugin)->close ();
	PLUGIN_OPS_INPUT(plugin)->seek (dvd_priv.title_start);
	PLUGIN_OPS_INPUT(plugin)->open (path);

	{	
		ifo_video_info_t *video;
		plugin_codec_attr_video_t attr;

		video = dvd_priv.ifo->data[ID_MAT] + IFO_OFFSET_VIDEO;

		// compression
		attr.mode = 0;	// VOB
		attr.fps = video->system;
		attr.aspect_mode = video->perm_displ;
		attr.aspect_output = video->ratio;
		attr.tv_system = -1;

		PLUGIN_OPS_CODEC(dvd_priv.plugin, set_attributes, BUF_VIDEO, &attr);
	}

	{
		ifo_audio_t *audio;
		plugin_codec_attr_audio_t attr;

// DENT: this sets the audio for the first stream - change again on stream change !!!  

		ifoGetAudio (dvd_priv.ifo->data[ID_MAT] + IFO_OFFSET_AUDIO, &audio);

		// audio += stream_nr;
//DENT: hmm, somethng is not as it should be (ifo parsing - bitfield)

		if (!audio->coding_mode && !audio->num_channels) {
			attr.mode = 2;	// LPCM
		} else {
			attr.mode = audio->coding_mode;
		}

		attr.bits = audio->quantization;
		attr.freq = audio->sample_freq ? 2 : 1;	// 96kHz : 48kHz
		attr.stream = 0;
		attr.volume = -1;

		PLUGIN_OPS_CODEC(dvd_priv.plugin, set_attributes, BUF_AUDIO, &attr);
	}
	
	{
	  char *pgci;
	  char *clut;
	  u_int num_cells;
	  plugin_codec_attr_spu_t attr;
	  ifo_pgc_cell_pos_t *cell_pos;

	  ifoGetPGCI ((ifo_hdr_t *) dvd_priv.ifo->data[ID_TITLE_PGCI], dvd_priv.title, &pgci),

	    // DENT a new CLUT starts in a pgc ... so move this out here
	  ifoGetCLUT (pgci, (char **) &clut);
	  attr.clut = clut;
	  attr.num  = 1;
	  //PLUGIN_OPS_CODEC(dvd_priv.plugin, set_attributes, BUF_SUBPIC, &attr);
	  // DENT end
	  
	  if ((num_cells = ifoGetCellPos (pgci, (u_char **) &cell_pos)) < 0) {
	    omsPrint (OMS_PRINT_ERROR, "ifoGetCellPos'ing\n");
	    exit (-1);
	  }

	  dvd_priv.num_cells = ifoGetCellAddrNum ((char *) dvd_priv.ifo->data[ID_TITLE_CELL_ADDR]);
	
	  cell_pos += dvd_priv.cur_pgci_cell_index;
	//dvd_priv.cell_vob_id = bswap_16 (cell_pos->vob_id);
	//dvd_priv.cell_cell_id = bswap_16 (cell_pos->cell_id);


	  omsPrint  (LOG_DEBUG, "num_cells: %d", dvd_priv.num_cells);
	}

	return DVD_PLAYER_OK;
}


/**
 *
 */

static int _dvd_PartPlayAutoStop (int title, int chapter, int parts2play)
{
	int num, i;
	char *ptr;
	char *pgci;
	ifo_cell_addr_t *ptt_cell_addr;

	dvd_priv.title = title;
	dvd_priv.chapter = chapter;


	ifoGetPGCI ((ifo_hdr_t *) dvd_priv.ifo->data[ID_TITLE_PGCI], dvd_priv.title, &pgci),
	num = ifoGetProgramMap (pgci, &ptr);

	ptr += dvd_priv.chapter;
	dvd_priv.cur_pgci_cell_index = *ptr - 1;

	// find the cell_index in the ptt cell table
	ptt_cell_addr = dvd_priv.ptr_cell_addr;
	for (i = 0; i < dvd_priv.num_cells; i++, ptt_cell_addr++) {
	  if (bswap_32(ptt_cell_addr->start) == bswap_32((dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index)->vobu_start))
	    break;
	}
	dvd_priv.cell_index = i;
	dvd_priv.cell_piece_index = dvd_priv.cell_index;
	omsPrint(LOG_DEBUG, "cell: start block 0x%08X\t end block 0x%08X\n", bswap_32((ptt_cell_addr + dvd_priv.cell_index)->start), bswap_32((ptt_cell_addr + dvd_priv.cell_index)->end));

	dvd_priv.next_pgci_cell_index = _dvd_next_cell_index();

	return _dvd_set_first_block ();

}


/**
 *
 */

static int _dvd_RegisterCb (u_int id, void (*cb) (void *, int ), void *data)
{
	omsPrint (LOG_DEBUG, "UI registered callback");

	switch (id) {
	case CB_CHANGE:
		dvd_priv.callb.chapter = cb;
		dvd_priv.callb.chapter_data = data;
		break;
	default:
		return -1;
	}

	return 0;	
}


/**
 *
 */

static int _dvd_UnregisterCb (u_int id)
{
	omsPrint (LOG_DEBUG, "UI unregistered callback %d", id);

	return 0;
}


/**
 *
 */

// "Title " + ?? + '\0'
#define TITLE_NAME_LEN (6 + 2 + 1)
// "Chapter " + ??? + '\0'
#define PART_NAME_LEN (8 + 3 + 1)

static nav_tree_t *_dvd_get_info (u_int id)
{
	nav_tree_t *tree = NULL;
        int i, s;

	switch (id) {
	case OMS_INFO_PROGRAM: {
		ifo_ptt_t *ptt;
#ifdef DVDDB
		FILE *dvddb_file;
		char dvd_id[9];
#endif

		if (!(ptt = ifo_get_ptt (dvd_priv.ifo)))
			return NULL;

		tree = malloc (sizeof (nav_tree_t));
		tree->num = ptt->num;
		tree->title = malloc (sizeof (nav_subtree_t) * tree->num);

#ifdef DVDDB
                if (dvddbGetID (dvd_priv.root, dvd_id) < 0) {
                        omsPrint (LOG_ERROR, "unable to get DVD-ID");
                        return NULL;
                }                                                                  

		if ((dvddb_file = dvddbOpen (dvd_id)) == NULL) {
			int answer;

			omsPrint (OMS_PRINT_DEBUG, "not found in local cache.");
			fprintf (stderr, "should i try to fetch it from the internet? (Y/n)");

			answer = getchar ();

			if (answer != 'n' && answer != 'N') {
				int sock;

				if ((sock = dvddbNetConnect ()) >= 0) {
					dvddbNetGet (sock, dvd_id);
					dvddbNetDisconnect (sock);

					if (!(dvddb_file = dvddbOpen (dvd_id))) {
omsPrint (OMS_PRINT_INFO, "--------------------------------------------------");
omsPrint (OMS_PRINT_INFO, "did not find an entry in the online-database,");
omsPrint (OMS_PRINT_INFO, "but it would really great f you submit this entry.");
omsPrint (OMS_PRINT_INFO, "(see src/FAQ: how do i submit new entries to the DVDDB?)");
omsPrint (OMS_PRINT_INFO, "- thanks in advance for your help");
omsPrint (OMS_PRINT_INFO, "--------------------------------------------------");
}
				}
			}
		}
#endif

		omsPrint (OMS_PRINT_DEBUG, "get_info: ptt_num: %d", ptt->num);

        	for (i=0; i<ptt->num; i++) {
                	ifo_ptt_sub_t *title = ptt->title+i;
			nav_subtree_t *sub_tree = tree->title+i;
			char *name;
#ifdef DVDDB
			char *name_ptr;
#endif

			sub_tree->sub = (nav_tree_data_t *) malloc (sizeof (nav_tree_data_t) * title->num);
			sub_tree->num = title->num;

#ifdef DVDDB
			if ((name_ptr = dvddbRead (dvddb_file))) {
				name = (char *) malloc (strlen (name_ptr) + 1);
				strcpy (name, name_ptr);
			} else
#endif
			{
				name = (char *) malloc (TITLE_NAME_LEN);
				snprintf (name, TITLE_NAME_LEN, "Title %d", i+1);
			}
			sub_tree->name = name;

#ifdef DVDDB
			dvddbRead (dvddb_file);		// read blank line after title
#endif
                	for (s=0; s<title->num; s++) {
				nav_tree_data_t *data = sub_tree->sub+s;
				char *name;

#ifdef DVDDB
				if ((name = dvddbRead (dvddb_file))) {
					name = (char *) malloc (strlen (name_ptr) + 1);
					strcpy (name, name_ptr);
				} else
#endif
				{
					name  = (char *) malloc (PART_NAME_LEN);
					snprintf (name, PART_NAME_LEN, "Chapter %d", s+1);
				}
				data->name = name;
                	}
#ifdef DVDDB
//		dvddbClose (dvddb_file);
#endif
        	}
		break;
	}
	case OMS_INFO_AUDIO: {
		ifo_audio_t *audio;
		int num;
		int i;

		if ((num = ifoGetAudio (dvd_priv.ifo->data[ID_MAT] + IFO_OFFSET_AUDIO, (char **) &audio)) <= 0)
			return NULL; 

		tree = malloc (sizeof (nav_tree_t));
		tree->num = num;
		tree->title = malloc (sizeof (nav_subtree_t) * tree->num);

		for (i=0; i<num; i++) {
			nav_subtree_t *sub_tree = tree->title+i;

			sub_tree->sub = NULL;
			sub_tree->num = 0;

			sub_tree->name = strdup (ifoDecodeLang (audio->lang_code));
			audio++;
		}
		
		break;
	}
	case OMS_INFO_SUBPIC: {
		ifo_spu_t *spu;
		int num;
		int i;

		if ((num = ifoGetSPU (dvd_priv.ifo->data[ID_MAT] + IFO_OFFSET_SUBPIC, (char **) &spu)) <= 0)
			return NULL; 

		tree = malloc (sizeof (nav_tree_t));
		tree->num = num;
		tree->title = malloc (sizeof (nav_subtree_t) * tree->num);

		for (i=0; i<num; i++) {
			nav_subtree_t *sub_tree = tree->title+i;

			sub_tree->sub = NULL;
			sub_tree->num = 0;

			sub_tree->name = strdup (ifoDecodeLang (spu->lang_code));
			spu++;
		}
		
		break;
	}
	default:
		omsPrint (OMS_PRINT_DEBUG, "unhandled OMS_INFO %d", id);
	}

	return tree;
}

 
/**
 * Initialize Information files
 */

static int _dvd_ifo_init ()
{
	u_char *ptr;
	u_int offset;
	u_int num_cells;
	//ifo_toast_t *ptt_info;

	if (!(dvd_priv.title_start = UDFFindFile (FILE_IFO))) {
		omsPrint (OMS_PRINT_DEBUG, "unable to UDFFindFile %s", FILE_IFO);
		return -1;
	}

// opening VIDEO_TS.IFO
	if (!(dvd_priv.ifo = ifoOpen (dvd_priv.fd_udf, dvd_priv.title_start * DVD_VIDEO_LB_LEN))) 
		return (-1);

// getting offset of first important ifo file
	ifoGetVMGPTT ((ifo_hdr_t *) dvd_priv.ifo->data[ID_PTT], (char **) &ptr);
	ptr += 8;

// DENT: check endian
        offset  = *(ptr+0) << 24;
        offset |= *(ptr+1) << 16;
        offset |= *(ptr+2) << 8;
        offset |= *(ptr+3);

//	omsPrint (OMS_PRINT_DEBUG, "IFO offset: 0x%x", offset);

	dvd_priv.title_start += offset;
	//ptt_ifo = (ifo_toast_t *)((ifo_hdr_t *)(ifo->data[ID_TSP]) + 1);
	//dvd_priv.seq_pgc = ptt_ifo->pgc_seq;
	//dvd_priv.rnd_prg = ptt_ifo->pgc_rnd;
	
// closing VIDEO_TS.IFO
	ifoClose (dvd_priv.ifo);

//DENT: we can add here a switch for (raw device or file system)
 
// opening VTS_??_0.IFO
	if (!(dvd_priv.ifo = ifoOpen (dvd_priv.fd_udf, dvd_priv.title_start * DVD_VIDEO_LB_LEN))) {
		omsPrint (OMS_PRINT_ERROR, "error initializing ifo");
		exit (-1);
	}

	dvd_priv.title_start += ifoGetVOBStart (dvd_priv.ifo);

	dvd_priv.num_cells = ifoGetCellAddr ((char *) dvd_priv.ifo->data[ID_TITLE_CELL_ADDR], (char **) &dvd_priv.ptr_cell_addr);
	
	ifoGetPGCI((ifo_hdr_t*) dvd_priv.ifo->data[ID_TITLE_PGCI], dvd_priv.title, &ptr);
	dvd_priv.num_pgci_cells=ifoGetCellPlayInfo(ptr, &dvd_priv.ptr_pgci_cell_addr);

	omsPrint(OMS_PRINT_DEBUG, "ifo init: number of programs %d", dvd_priv.num_pgci_cells);

#if DEBUG
	ifoPrintCellInfo(dvd_priv.ptr_pgci_cell_addr, dvd_priv.num_pgci_cells);
#endif 

	return 0;
}


/**
 *
 */

static int _dvd_next_cell_index (void)
{
	ifo_pgci_cell_addr_t *cell_addr;
	int index;

// find cell piece
	cell_addr = dvd_priv.ptr_pgci_cell_addr;
	cell_addr += dvd_priv.cur_pgci_cell_index;

	omsPrint(LOG_DEBUG, "chain_info 0x%x\tfoo: 0x%x", cell_addr->chain_info, cell_addr->foo);
	omsPrint(LOG_DEBUG, "still_time: %02xs\tcell_cmd: 0x%x\tlen: %02x:%02x:%02x:%02x %sframes/s",
		 cell_addr->still_time & 0xFF,
		 cell_addr->cell_cmd & 0xFF,
		 (bswap_32(cell_addr->len_time) >> 24) & 0xFF,
		 (bswap_32(cell_addr->len_time) >> 16) & 0xFF,
		 (bswap_32(cell_addr->len_time) >> 8) & 0xFF,
		 bswap_32(cell_addr->len_time) & 0x3F, bswap_32(cell_addr->len_time) & 0x80 ? "30" : "25 non-drop");
	omsPrint(LOG_DEBUG, "1st_vobu_start: 0x%08X 1st_ilvu_end: 0x%08X", bswap_32(cell_addr->vobu_start), bswap_32(cell_addr->ilvu_end));
	omsPrint(LOG_DEBUG, "last_vobu_start: 0x%08X last_vobu_end: 0x%08X", bswap_32(cell_addr->vobu_last_start), bswap_32(cell_addr->vobu_last_end));
	omsPrint(LOG_DEBUG, "current cell index: %d\tcell piece index %d", dvd_priv.cell_index, dvd_priv.cell_piece_index);
	
	if ((cell_addr->chain_info & 0xff) == 0x5e) { // multiangle non selectable, find next single angle cell
	  for (index=dvd_priv.cur_pgci_cell_index;index<dvd_priv.num_cells; index++, cell_addr++) {
		omsPrint(LOG_DEBUG, "chain_info: 0x%x", cell_addr->chain_info);
		if ((cell_addr->chain_info & 0xf) != 0xe)
		  break;
	  }
	  goto multiangle;
	} else if ((cell_addr->chain_info & 0xff) == 0x5f) { // multiangle, selectable
	  for (index=dvd_priv.cur_pgci_cell_index;index<dvd_priv.num_cells; index++, cell_addr++) {
		omsPrint(LOG_DEBUG, "chain_info: 0x%x", cell_addr->chain_info);
		if ((cell_addr->chain_info & 0xf) != 0xf)
		  break;
	  }
	  goto multiangle;
	} else if ((cell_addr->chain_info & 0xff) == 0xe) { // non equivalent interleave
		omsPrint(LOG_DEBUG, "non equivalent interleave\n");
		for (index=dvd_priv.cur_pgci_cell_index;index<dvd_priv.num_cells; index++, cell_addr++) {
		  omsPrint(LOG_DEBUG, "chain_info: 0x%x", cell_addr->chain_info);
		  if ((cell_addr->chain_info & 0xff) < 0xc)
		    break;
		}
		goto non_eqlv_ilv;
	} else if ((u_char)(cell_addr->chain_info & 0xff) > 0xc) {
		fprintf(stderr, "oms: unknown ifo pgc interleave mode. \n
				please turn on the debug and send the output
				to livid-dev@linuxvideo.org.  Thanks. \n");
		exit(1);
	} else {
	        index = dvd_priv.cur_pgci_cell_index + 1;  
	}

	omsPrint(LOG_DEBUG, "setting num angles back to 1");

	dvd_priv.non_equ_ilv = 0;
	dvd_priv.num_angles = 1;
	dvd_priv.cells_per_angle = 1;
	dvd_priv.last_angle_block = bswap_32((dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index)->vobu_last_end);
	dvd_priv.next_cell_index = dvd_priv.cell_index + 1;
	dvd_priv.cell_piece_index = dvd_priv.cell_index;

	omsPrint(LOG_DEBUG, "1 angle\tcurrent pgci_cell_index %d\tnext_pgci__cell_index: %d", dvd_priv.cur_pgci_cell_index, dvd_priv.next_pgci_cell_index);
	omsPrint(LOG_DEBUG, "1 angle\tcurrent cell_index %d\tnext_cell_index: %d\tcell_piece_index: %d", dvd_priv.cell_index, dvd_priv.next_cell_index, dvd_priv.cell_piece_index);

	return index;
	
 multiangle:
	
	dvd_priv.non_equ_ilv = 0;
	dvd_priv.num_angles = index - dvd_priv.cur_pgci_cell_index;
	// find out how many cells are in the angle
	{
	  u_int i = 1;
	  ifo_cell_addr_t *ptt_cell_addr = dvd_priv.ptr_cell_addr + dvd_priv.cell_index;
	  while (bswap_32(ptt_cell_addr->end) != bswap_32((dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index)->vobu_last_end)) {
	    i++;
	    ptt_cell_addr++;
	  }
	  dvd_priv.cells_per_angle = i;
	}
	dvd_priv.cur_pgci_cell_index += dvd_priv.angle;
	dvd_priv.last_angle_block = bswap_32((dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index)->vobu_last_end);
	dvd_priv.next_cell_index = dvd_priv.cell_index + dvd_priv.num_angles * dvd_priv.cells_per_angle;
	dvd_priv.cell_index += dvd_priv.cells_per_angle * dvd_priv.angle;
	dvd_priv.cell_piece_index = dvd_priv.cell_index;

	omsPrint(LOG_DEBUG, "%d angles\tcells per angle: %d\tcurrent cell_index %d\tnext_cell_index: %d\tcell_piece_index: %d", dvd_priv.num_angles, dvd_priv.cells_per_angle, dvd_priv.cell_index, dvd_priv.next_cell_index, dvd_priv.cell_piece_index);
	omsPrint(LOG_DEBUG, "current pgci_cell_index %d\tnext_pgci_cell_index: %d", dvd_priv.cur_pgci_cell_index, dvd_priv.next_pgci_cell_index);

	return index;

 non_eqlv_ilv:
	
	// reset angle stuff
	dvd_priv.num_angles = 1;
	dvd_priv.cells_per_angle = 1;

	// find interleaved cell numbers
	dvd_priv.non_equ_ilv = 1;
	dvd_priv.ilv_pg_index = 0;
	dvd_priv.num_ilv_pgms = index - dvd_priv.cur_pgci_cell_index;
	dvd_priv.cell_index = dvd_priv.next_cell_index;
	dvd_priv.cell_piece_index = dvd_priv.cell_index;

	omsPrint(LOG_DEBUG, "num_ilv_pgms %d", dvd_priv.num_ilv_pgms);

	{
	  u_int i = 1, k;
	  u_int total_cells = 0;
	  ifo_cell_addr_t *ptt_cell_addr = dvd_priv.ptr_cell_addr + dvd_priv.cell_index;
	  for (k = 0; k < dvd_priv.num_ilv_pgms; k++, dvd_priv.cur_pgci_cell_index++) {
	    i = 1;
	    while (bswap_32(ptt_cell_addr->end) < bswap_32((dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index)->vobu_last_end)) {
	      i++;
	      ptt_cell_addr++;
	    }
	    if (bswap_32(ptt_cell_addr->end) == bswap_32((dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index)->vobu_last_end)) {
	      dvd_priv.jump_cell[k] = 0;
	    } else if (bswap_32(ptt_cell_addr->end) > bswap_32((dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index)->vobu_last_end)) {
	      // vobu interleave doesn't match cell boundary, jump on cell
	      dvd_priv.jump_cell[k] = 1;
	    } else {
	      omsPrint(LOG_DEBUG, "shouldn't be here");
	      exit(1);
	    }
	    
	    ptt_cell_addr++; // consume the current cell
	    dvd_priv.num_ilv_cell[k] = i;
	    total_cells += i;
	    omsPrint(LOG_DEBUG, "num_ilv_cell %d: %d\t%s", k, i, dvd_priv.jump_cell[k] == 1 ? "jump" : "no jump");
	  }
	  
	  dvd_priv.next_cell_index += total_cells * dvd_priv.num_ilv_pgms;
	  omsPrint(LOG_DEBUG, "total non equ ilv cells %d", total_cells);
	  }
	
	return index;	     
}


/**
 *
 */

static int _dvd_set_first_block ()
{
	ifo_pgci_cell_addr_t *pgci_cell_addr;
	int index;

        omsPrint (LOG_DEBUG, "******************** 1stblock 1");                   
	
	pgci_cell_addr = dvd_priv.ptr_pgci_cell_addr + dvd_priv.cur_pgci_cell_index;

	dvd_priv.block = bswap_32 (pgci_cell_addr->vobu_start);
	dvd_priv.last_angle_block = bswap_32(pgci_cell_addr->vobu_last_end);
	omsPrint(LOG_DEBUG, "block 0x%x\tlast_angle_block: 0x%08x\n", dvd_priv.block, dvd_priv.last_angle_block);
	PLUGIN_OPS_INPUT(dvd_priv.plugin)->seek (dvd_priv.block + dvd_priv.title_start);

	omsPrint(LOG_DEBUG, "title_start 0x%x\n", dvd_priv.title_start);

        omsPrint (LOG_DEBUG, "******************** 1stblock 2");                   
	return 0;
}


/**
 *
 */

static int _dvd_next_block (char *data, u_int len)
{
	ifo_pgci_cell_addr_t *pgci_cell_addr;
	ifo_cell_addr_t *cell_addr;
	u_int cells_sofar;

	pgci_cell_addr = dvd_priv.ptr_pgci_cell_addr;
	pgci_cell_addr += dvd_priv.cur_pgci_cell_index;
	cell_addr = dvd_priv.ptr_cell_addr;
	
	// end of title
	if (dvd_priv.cur_pgci_cell_index == dvd_priv.num_pgci_cells)
	   return -1;

	cells_sofar = dvd_priv.cell_index - dvd_priv.cell_piece_index;
	if (dvd_priv.non_equ_ilv == 0) {
	  if (cells_sofar < dvd_priv.cells_per_angle - 1) {
	    if (dvd_priv.block > bswap_32((cell_addr + dvd_priv.cell_index)->end)) {
	      dvd_priv.cell_index++;
	      omsPrint(LOG_DEBUG, "cell: start block 0x%08x\t end block 0x%08X", bswap_32((cell_addr + dvd_priv.cell_index)->start), bswap_32((cell_addr + dvd_priv.cell_index)->end));
	      dvd_priv.block = bswap_32((cell_addr + dvd_priv.cell_index)->start);
	      PLUGIN_OPS_INPUT(dvd_priv.plugin)->seek (dvd_priv.block + dvd_priv.title_start);
	    } else {
	      goto read_block;
	    }
	  } else if (cells_sofar == dvd_priv.cells_per_angle - 1) {
	    if (dvd_priv.block > dvd_priv.last_angle_block) { // next cell_piece
	      dvd_priv.cur_pgci_cell_index = dvd_priv.next_pgci_cell_index;
	      dvd_priv.cell_index = dvd_priv.next_cell_index;
	      dvd_priv.next_pgci_cell_index = _dvd_next_cell_index();
	      cell_addr += dvd_priv.cell_index;
	      omsPrint(LOG_DEBUG, "cell: start block 0x%08x\t end block 0x%08X", bswap_32(cell_addr->start), bswap_32(cell_addr->end));
	      
	      dvd_priv.block = bswap_32(cell_addr->start);
	      PLUGIN_OPS_INPUT(dvd_priv.plugin)->seek (dvd_priv.block + dvd_priv.title_start);
	      
	      omsPrint(LOG_DEBUG, "next pgci cell index: %d", dvd_priv.next_pgci_cell_index);
	      
	    } else {
	      goto read_block;
	    }
	  }
	} else { // non equivalent interleave
	  if (dvd_priv.ilv_pg_index < dvd_priv.num_ilv_pgms - 1) {
	    if (cells_sofar < dvd_priv.num_ilv_cell[dvd_priv.ilv_pg_index] - 1) {
	      if (dvd_priv.block > bswap_32((cell_addr + dvd_priv.cell_index)->end)) {
		dvd_priv.cell_index++;
		
		omsPrint(LOG_DEBUG, "cell: start block 0x%08x\t end block 0x%08X", bswap_32((cell_addr + dvd_priv.cell_index)->start), bswap_32((cell_addr + dvd_priv.cell_index)->end));
		dvd_priv.block = bswap_32((cell_addr + dvd_priv.cell_index)->start);
		PLUGIN_OPS_INPUT(dvd_priv.plugin)->seek (dvd_priv.block + dvd_priv.title_start);
	      } else {
		goto read_block;
	      }
	    } else if (cells_sofar == dvd_priv.num_ilv_cell[dvd_priv.ilv_pg_index] - 1) {
	      if (dvd_priv.block > bswap_32((cell_addr + dvd_priv.cell_index)->end)) { // next interleaved program
		
		dvd_priv.cell_index++;
		dvd_priv.cell_piece_index = dvd_priv.cell_index;
		if (dvd_priv.jump_cell[dvd_priv.ilv_pg_index]){ // jump over repeated cell
		  dvd_priv.cell_index++;
		  omsPrint(LOG_DEBUG, "cell jump");
		}

		dvd_priv.ilv_pg_index++;
		
		cell_addr += dvd_priv.cell_index;
		omsPrint(LOG_DEBUG, "cell: start block 0x%08x\t end block 0x%08X", bswap_32(cell_addr->start), bswap_32(cell_addr->end));
		
		dvd_priv.block = bswap_32(cell_addr->start);
		PLUGIN_OPS_INPUT(dvd_priv.plugin)->seek (dvd_priv.block + dvd_priv.title_start);
		
	      } else {
		goto read_block;
	      }
	    }
  
	} else if (dvd_priv.ilv_pg_index == dvd_priv.num_ilv_pgms - 1) {
	    if (cells_sofar < dvd_priv.num_ilv_cell[dvd_priv.ilv_pg_index] - 1) {
	      if (dvd_priv.block > bswap_32((cell_addr + dvd_priv.cell_index)->end)) {
		dvd_priv.cell_index++;
		
		omsPrint(LOG_DEBUG, "cell: start block 0x%08x\t end block 0x%08X", bswap_32((cell_addr + dvd_priv.cell_index)->start), bswap_32((cell_addr + dvd_priv.cell_index)->end));
		dvd_priv.block = bswap_32((cell_addr + dvd_priv.cell_index)->start);
		PLUGIN_OPS_INPUT(dvd_priv.plugin)->seek (dvd_priv.block + dvd_priv.title_start);
	      } else {
		goto read_block;
	      }
	    } else if (cells_sofar == dvd_priv.num_ilv_cell[dvd_priv.ilv_pg_index] - 1) {
	      // last cell of the last interleaved program
	      if (dvd_priv.block > bswap_32((cell_addr + dvd_priv.cell_index)->end)) {
		dvd_priv.cur_pgci_cell_index = dvd_priv.next_pgci_cell_index;
		dvd_priv.cell_index = dvd_priv.next_cell_index;
		dvd_priv.next_pgci_cell_index = _dvd_next_cell_index();
		cell_addr += dvd_priv.cell_index;
		omsPrint(LOG_DEBUG, "last non equl ilv pg\t cell: start block 0x%08x\t end block 0x%08X", bswap_32(cell_addr->start), bswap_32(cell_addr->end));
	      
		dvd_priv.block = bswap_32(cell_addr->start);
		PLUGIN_OPS_INPUT(dvd_priv.plugin)->seek (dvd_priv.block + dvd_priv.title_start);
	      
		omsPrint(LOG_DEBUG, "next pgci cell index: %d", dvd_priv.next_pgci_cell_index);
	      
	      } else {
		goto read_block;
	      }	    
	    }
	}
	}

	// chapter change
 	{
 	  char *ptr; 
 	  char *pgci; 

 	  ifoGetPGCI ((ifo_hdr_t *) dvd_priv.ifo->data[ID_TITLE_PGCI], dvd_priv.title, &pgci);

 	  if (dvd_priv.chapter < ifoGetProgramMap (pgci, &ptr))
 	    ptr += dvd_priv.chapter + 1; 
	  
 	  if (dvd_priv.cur_pgci_cell_index == (*ptr - 1)) { 
 	    dvd_priv.chapter ++; 
 	    if (dvd_priv.callb.chapter) 
 	      dvd_priv.callb.chapter (dvd_priv.callb.chapter_data, dvd_priv.chapter); 
	    
 	  } 
	  
 	} 

	read_block:
	
	//fprintf(stderr, "block number: 0x%x\ttitle_start 0x%x\n", dvd_priv.block, dvd_priv.title_start);
	  dvd_priv.block++;	
	  return PLUGIN_OPS_INPUT(dvd_priv.plugin)->read (data, len);
}


/**
 * Initialize plugin.
 **/

void *plugin_init (char *whoami)
{
	pluginRegister (whoami,
		PLUGIN_ID_NAV,
		0,
		&dvd_nav);

	return &dvd_nav;
}


/**
 *
 **/

void plugin_exit (void)
{
}


