#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

#include <oms/oms.h>
#include "oms/buf.h"
#include <oms/plugin.h>
#include <oms/plugin/nav.h>

#include "../input/input_vcd.h"

static int _vcd_open (plugin_t *plugin, char *path);
static int _vcd_PartPlayAutoStop (int title, int chapter, int parts2play);
static int _vcd_RegisterCb (u_int id, void (*cb) (void *, int ), void *data);
static int _vcd_UnregisterCb (u_int id);
static nav_tree_t *vcdGetInfo (u_int id);

static struct {
	char *root;
	plugin_t *nav;
} vcd_priv;

static plugin_nav_t vcd_nav = {
	_vcd_open,
	NULL,
	_vcd_PartPlayAutoStop,
	vcdGetInfo,
	_vcd_RegisterCb,		// register callback
	_vcd_UnregisterCb,
	&vcd_priv
};


/**
 *
 */

static int _vcd_open (plugin_t *plugin, char *path)
{
	vcd_priv.root = path;
	vcd_priv.nav = plugin;

	if (pluginOpen (plugin, PLUGIN_ID_INPUT, DL_INPUT_VCD, NULL))
		return -1;

	if (pluginOpen (plugin, PLUGIN_ID_DECAPS, DL_DECAPS_VCD, NULL))
		return -1;

	PLUGIN_OPS_DECAPS(plugin)->blocksize = PLUGIN_OPS_INPUT(plugin)->blocksize;
	PLUGIN_OPS_INPUT(plugin)->open (path);
	PLUGIN_OPS_DECAPS(plugin)->read = PLUGIN_OPS_INPUT(plugin)->read;

	if (!plugin->ops[PLUGIN_ID_CODEC]) {
		omsPrint (OMS_PRINT_DEBUG, "no codec registered");
		return -1;
	} else
		PLUGIN_OPS_DECAPS(plugin)->write = demuxBufPut;

//	PLUGIN_OPS_INPUT(plugin)->seek (0);

	PLUGIN_OPS_NAV(plugin) = &vcd_nav;

        return DVD_PLAYER_OK;
}


/**
 *
 */

static int _vcd_PartPlayAutoStop (int title, int chapter, int parts2play)
{
	plugin_codec_attr_audio_t attr_audio = {1, 0, 0, 0, -1};
	plugin_codec_attr_video_t attr_video = {1, 0, 1, -1, 2};

	PLUGIN_OPS_INPUT(vcd_priv.nav)->seek (chapter);	

//VCD MPEG,50Hz, PAL
	PLUGIN_OPS_CODEC(vcd_priv.nav, set_attributes, BUF_VIDEO, &attr_video);
//MPEG,16bit,44.1kHz
	PLUGIN_OPS_CODEC(vcd_priv.nav, set_attributes, BUF_AUDIO, &attr_audio);

	return DVD_PLAYER_OK;
}


/**
 *
 */

static int _vcd_RegisterCb (u_int id, void (*cb) (void *, int ), void *data)
{
	omsPrint (OMS_PRINT_DEBUG, "UI registered callback %s", "tjo");

	switch (id) {
	case CB_CHANGE:
		((vcd_priv_t*)(PLUGIN_OPS_INPUT(vcd_priv.nav)->priv))->callb.chapter = cb;
		((vcd_priv_t*)(PLUGIN_OPS_INPUT(vcd_priv.nav)->priv))->callb.chapter_data = data;
		break;
	}

	return 0;
}


/**
 *
 */

static int _vcd_UnregisterCb (u_int id)
{
	return 0;
}


/**
 *
 */

#define TRACK_NAME_LEN (6 + 3 + 1)

static nav_tree_t *vcdGetInfo (u_int id)
{
	nav_tree_t *tree=0;
	int i;

	switch (id) {
	case OMS_INFO_PROGRAM: {
		int tracks;

		tracks=((vcd_priv_t*)(PLUGIN_OPS_INPUT(vcd_priv.nav)->priv))->total_tracks;
		tree = malloc (sizeof (nav_tree_t));

        	tree->num = 1;
        	tree->title = malloc (sizeof (nav_subtree_t) * tree->num);

		{
			nav_subtree_t *sub_tree = tree->title;
			char *name;

			sub_tree->sub = (nav_tree_data_t *) malloc (sizeof
			(nav_tree_data_t) * tracks);

			sub_tree->num = tracks;
			sub_tree->name = "VCD";
			for (i=0; i < tracks; i++) {
				nav_tree_data_t *data = sub_tree->sub+i;
				name = (char *) malloc (TRACK_NAME_LEN);
				snprintf (name, TRACK_NAME_LEN, "Track %d",i);
				data->name = name;
			}
		}
		break;
	}
	default:
		omsPrint (OMS_PRINT_DEBUG, "unhandled OMS_INFO %s", "tjo");
	}

	return tree;
}


/**
 *
 **/

void *plugin_init (char *whoami)     
{
	pluginRegister (whoami,
		PLUGIN_ID_NAV,
		0,
		&vcd_nav);

        return &vcd_nav;     
}     


/**
 *
 */

void plugin_exit (void)
{
}

