/*  Open Media System - oms
 * 
 *  devices page in configuration dialog
 *
 * Copyright (C) 1999  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config_store.h"

#include "config_devices.h"
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

GtkWidget *devlist, *deventry ;
gint      nSelectedRow;
GList     *pGLdev;

GList *get_dev_list ()
{
  return pGLdev;
}

void get_devices_config () 
{
  gchar *strDevices ;
  gchar **strdevlist;
  gchar *str;
  int i;

  /* 
   * empty devlist
   */

  while ((str=(gchar *) g_list_nth_data (pGLdev, 0)))
    {
      free (str);
      g_list_remove (pGLdev, str);
    }

  /*
   * build new list from configfile entry
   */

  pGLdev = NULL;

  strDevices = cfg_get_val_string ("devices");

  if (strDevices == NULL) {
    strDevices = "/dev/dvd";
  }

  strdevlist = g_strsplit (strDevices, ",", 0);

  i = 0;
  while ((str = strdevlist[i]))
    {
      fprintf (stderr, "device : %s\n",str);
      pGLdev = g_list_append (pGLdev, g_strdup(str));
      i++;
    }

  g_strfreev  (strdevlist);
}

/* 
 * copy pGLdev to config manager
 */

void config_device_apply ()
{
  gchar *str;
  GString *pdevstring ;
  int i;

  pdevstring = g_string_new ("");

  i = 0;
  while ((str = (gchar *) g_list_nth_data(pGLdev,i)))
    {
      if (pdevstring->len>0)
	g_string_append_c (pdevstring, ',');
      g_string_append (pdevstring, str);
      i++;
    }
  cfg_set_val_string ("devices", pdevstring->str);

  fprintf (stderr, "config_devices_apply : devices = %s\n", pdevstring->str);

  g_string_free (pdevstring, FALSE);
}

/*
 * copy pGLdev to GUI clist
 */
void setup_devlist () 
{
  gchar *strlist[2] = { NULL, NULL } ;
  gchar *str;
  int i;

  gtk_clist_clear ((GtkCList *) devlist);

  i = 0;
  while ((str = (gchar *) g_list_nth_data(pGLdev,i)))
    {
      fprintf (stderr, "GUI device : %s\n",str);
      strlist[0] = str;
      gtk_clist_append( (GtkCList *) devlist, (gchar **) &strlist);
      i++;
    }
}

void remove_device (GtkButton *button,
		    gpointer user_data) {


  if (nSelectedRow >=0) 
    {
      g_list_remove (pGLdev, g_list_nth_data (pGLdev, nSelectedRow));

      setup_devlist();

      nSelectedRow = 0;

    }

}

void selection_made(GtkWidget *widget,
		    gint row,
		    gint column,
		    GdkEventButton *event,
		    gpointer data)
{
  nSelectedRow = row;
}


void add_device (GtkButton *button,
		 gpointer user_data) {
  gchar *strDev;
  struct stat stat_buffer;

  strDev = gtk_entry_get_text ((GtkEntry *) deventry);

  if (strlen(strDev)>0) 
    {
      if (stat (strDev, &stat_buffer)==0)
	{
	  if (S_ISCHR (stat_buffer.st_mode) 
	      || S_ISBLK (stat_buffer.st_mode))
	    {
	      g_list_append (pGLdev, g_strdup (strDev));
	      setup_devlist();
	      fprintf (stderr, "adding device %s \n",strDev);
	    }
	}

    }
}

GtkWidget *create_devices_page ()
{

  GtkWidget *vbox, *hbox, *addButton, *delButton, *scroller ;

  pGLdev = NULL;

  nSelectedRow = -1;

  vbox = gtk_vbox_new (FALSE, 5);
  gtk_widget_show (vbox);

  /*
   * the List
   */

  scroller = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
				  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_AUTOMATIC);
  gtk_widget_set_usize (scroller, 150, 200);
  gtk_box_pack_start (GTK_BOX (vbox), scroller, TRUE, TRUE, 5);
  gtk_widget_show (scroller);
  
  devlist = gtk_clist_new (1);
  gtk_widget_show (devlist);

  gtk_signal_connect_object (GTK_OBJECT (devlist), "select_row",
			     GTK_SIGNAL_FUNC(selection_made),
			     NULL);
  
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scroller), devlist);

  /*
   * buttons
   */
  


  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
  gtk_widget_show(hbox);

  deventry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hbox), deventry, TRUE, TRUE, 5);
  gtk_widget_show(deventry);

  addButton = gtk_button_new_with_label ("Add Device");

  
  gtk_signal_connect_object (GTK_OBJECT (addButton), "clicked",
			     (GtkSignalFunc) add_device,
			     NULL);
  

  gtk_box_pack_start (GTK_BOX (hbox), addButton, TRUE, TRUE, 5);
  gtk_widget_show(addButton);

  delButton = gtk_button_new_with_label ("Remove Device");
  
  gtk_signal_connect_object (GTK_OBJECT (delButton), "clicked",
			     (GtkSignalFunc) remove_device,
			     NULL); 
  
  gtk_box_pack_start (GTK_BOX (hbox), delButton, TRUE, TRUE, 5);
  gtk_widget_show(delButton);


  get_devices_config ();
  setup_devlist () ;

  return vbox;

}





