/*  Open Media System - oms
 * 
 *  plugins configuration page
 *
 *  Copyright (C) 2000  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config_store.h"

#include "config_plugins.h"
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <dirent.h>
#include <unistd.h>
#include <config.h>

GtkWidget *pPluginTable;

GList *plugin_getConfigOptions ()
{
    GList           *pList = NULL;
    config_plugin_t *pOption;

    pOption = malloc (sizeof (config_plugin_t));
    pOption->strName = "Start fullsreen?";
    pOption->nType   = PTYPE_BOOLEAN;
    pOption->strHelp = "Should oms start in fullscreen mode?";
    pList = g_list_append (pList, pOption);

    pOption = malloc (sizeof (config_plugin_t));
    pOption->strName  ="Color"; 
    pOption->nType    = PTYPE_REAL;
    pOption->bLimited = TRUE;
    pOption->fMin     = 0.0;
    pOption->fMax     = 1.0;
    pOption->strHelp  ="Color saturation"; 
    pList = g_list_append (pList, pOption);

    pOption = malloc (sizeof (config_plugin_t));
    pOption->strName = "Window title";
    pOption->nType   = PTYPE_STRING;
    pOption->strHelp  ="Enter desired title for oms video window"; 
    pList = g_list_append (pList, pOption);
    
    return pList;
}

void config_plugins_apply ()
{
    /*
      gchar *str_skin=NULL; 
      
      
      gtk_clist_get_text  (GTK_CLIST(skinlist), nSelectedRow, 0, &str_skin);
      
      set_config_string_value ("skin", str_skin);
      
      load_skin (str_skin);
      apply_panel_skin ();
    */
}

void cb_pluginselected (GtkWidget *item, gchar *signame)
{
    GList           *old_widgets;
    GList           *options;
    config_plugin_t *pOption;
    int              i;
    GtkWidget       *label, *widget;
    GtkObject       *adj;

    /*
     * clear table
     */

    old_widgets = gtk_container_children (GTK_CONTAINER (pPluginTable));

    while (old_widgets)
	{
	    gtk_container_remove (GTK_CONTAINER (pPluginTable),
				  GTK_WIDGET (old_widgets->data));
	    old_widgets = g_list_next (old_widgets);
	}

    /*
     * get config information for plugin
     */

    options = plugin_getConfigOptions () ;
    
    /*
     * build gui for it
     */

    i=0;
    while (options)
	{
	    pOption = (config_plugin_t *) options->data;

	    if (pOption->nType != PTYPE_BOOLEAN)
		{
		    label = gtk_label_new (pOption->strName);
		    gtk_table_attach( GTK_TABLE (pPluginTable),
				      label,
				      0, 1, i, i+1,
				      0, 0,
				      5, 5);
		    
		    gtk_widget_show (label);
		}
	    
	    
	    switch (pOption->nType)
		{
		case PTYPE_STRING:
		    widget = gtk_entry_new ();
		    break;
		case PTYPE_BOOLEAN:
		    widget = gtk_check_button_new_with_label(pOption->strName);
		    break;
		case PTYPE_REAL:
		    adj = gtk_adjustment_new( 0.0,
					      pOption->fMin,
					      pOption->fMax,
					      0.1,
					      1.0,
					      0.1);
		    widget = gtk_hscale_new (adj);
		    break;
		default:
		    widget = NULL;
		}

	    if (widget)
		{
		    gtk_table_attach( GTK_TABLE (pPluginTable),
				      widget,
				      1, 2, i, i+1,
				      0, 0,
				      5, 5);
 
		    gtk_widget_show (widget);
		}

	    options = g_list_next (options);
	    i++;
	}
}

/*
 * scan plugin directory for plugins
 */
GtkWidget *scanPlugins (char *strDir_) 
{
    DIR           *plugindir;
    struct dirent *direntry;
    GtkWidget     *pTree;
    
    plugindir = opendir (strDir_);
    
    pTree = gtk_tree_new ();
    gtk_widget_show (pTree);
    
    if (!plugindir) 
	{
	    GtkWidget *pItem ;

	    pItem = gtk_tree_item_new_with_label( "error" );
	    
	    gtk_tree_append (GTK_TREE(pTree), pItem);
	    
	    return pTree;
	}

    while ((direntry = readdir (plugindir)) != NULL)
	{
	    if ((strcmp (direntry->d_name,"."))
		&& (strcmp (direntry->d_name,"..")))
		{
		    struct stat stat_buffer;
		    char pathname[256];
		    int len = strlen (direntry->d_name);
		    
		    sprintf (pathname, "%s/%s", strDir_, direntry->d_name); 

		    stat (pathname, &stat_buffer);

		    if (S_ISDIR (stat_buffer.st_mode))
			{
			    GtkWidget *pItem ;

			    pItem = gtk_tree_item_new_with_label(direntry->d_name);
			    gtk_widget_show (pItem);
			    gtk_tree_append (GTK_TREE(pTree), pItem);

			    gtk_tree_item_set_subtree( GTK_TREE_ITEM(pItem),
						       scanPlugins (pathname));

			    gtk_tree_item_expand (GTK_TREE_ITEM(pItem));
			}
		    else if ((direntry->d_name[len-2]=='s')
			     && (direntry->d_name[len-1]=='o'))
			{
			    GtkWidget *pItem ;

			    pItem = gtk_tree_item_new_with_label(direntry->d_name);
			    gtk_widget_show (pItem);

			    gtk_tree_append (GTK_TREE(pTree), pItem);

			    gtk_signal_connect (GTK_OBJECT(pItem), "select",
						GTK_SIGNAL_FUNC(cb_pluginselected),
						"select");

			}

		}
	}

    closedir (plugindir);

    return pTree;
}


GtkWidget *create_plugins_page ()
{

    GtkWidget *hbox, *scroller, *plugintree, *pscroller ;

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_widget_show (hbox);

    /*
     * the Tree
     */

    scroller = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize (scroller, 150, 200);
    gtk_box_pack_start (GTK_BOX (hbox), scroller, FALSE, TRUE, 2);
    gtk_widget_show (scroller);
  
    plugintree = scanPlugins (OMS_PLUGINDIR);

    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scroller), 
					   plugintree);

    /*
     * room for plugin-specific configuration widgets
     */

    pscroller = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pscroller),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize (pscroller, 300, 100);
    gtk_box_pack_start (GTK_BOX (hbox), pscroller, TRUE, TRUE, 2);
    gtk_widget_show (pscroller);

    pPluginTable = gtk_table_new(15,
                                 2,
                                 FALSE);

    /*     pPluginVBox = gtk_vbox_new (FALSE, 2); */
    gtk_widget_show (pPluginTable);
  
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(pscroller), 
					   pPluginTable);
    return hbox;

}
