/* Open Media System - oms
 *
 * Copyright (C) 1999, 2000  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h> 
#include <pthread.h>

#include <gtk/gtk.h>

#include <config.h>

#ifndef HAVE_IMLIB2
#include <gdk_imlib.h>
#endif

#include <oms/oms.h>

#include "main.h"
#include "panel_gui.h"
#include "config_gui.h"
#include "config_store.h"
#include "playlist.h"
#include "skin.h"
#include "messagebox.h" 

static volatile pthread_t threadlock_owner = (pthread_t) 0;

struct player_global_struct player_global;
GtkWidget *widget_panel;
GtkWidget *widget_playlist;
GtkWidget *widget_config;


/**
 *
 **/

void oms_gtk_threads_enter(void)
{
	pthread_t this_thread = pthread_self();

	if (this_thread != threadlock_owner) {
		gdk_threads_enter();
		threadlock_owner = this_thread;
	}
}


/**
 *
 **/

void oms_gtk_threads_leave(void)
{
	pthread_t this_thread = pthread_self();

	if (this_thread != threadlock_owner) {
		threadlock_owner = (pthread_t) 0;
		gdk_threads_leave();
	}
}


/**
 *
 **/

int main (int argc, char *argv[])
{
        gchar *skinname;

	player_global.bPLShowing    = FALSE;
	player_global.bConfShowing  = FALSE;
  
	g_thread_init (NULL) ;
	gtk_init (&argc, &argv);

#ifndef HAVE_IMLIB2
	gdk_imlib_init ();
#endif

	cfg_init ();
	cfg_read ();
	
	skinname = cfg_get_val_string ("skin");
	if (!skinname || (load_skin (skinname)<0)) {
		if (load_skin ("default")==-1) {
			fprintf (stderr, "Couldn't find any skin\n");
			exit (-1);
		}
	}

	
	omsOpenOutput (cfg_get_defval_string ("outputplugin",
			OMS_PLUGINDIR"/codec_sw.so"),
		        cfg_get_defval_string ("outputdevice",
			"x11:oss"));

	widget_panel     = setup_panel ();
	widget_config    = setup_config ();
	widget_playlist  = setup_playlist ();

	THREADSAFE (gtk_widget_set_uposition (GTK_WIDGET(widget_panel), cfg_get_val_int ("xpos", 200), cfg_get_val_int ("ypos", 0)));

	gtk_widget_show (widget_panel);

	draw_panel_window(FALSE);
  
	THREADSAFE (gtk_main ());
  
	return 0;
}
