/*  Open Media System - oms
 *
 *  Copyright (C) 1999, 2000  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gtk/gtk.h"
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <stdio.h>
#include <gdk_imlib.h>

#include "skin.h"
#include "config_store.h"
#include "pbutton.h"
#include "panel_gui.h"
#include "playlist.h"
#include "main.h"

#include "oms/oms.h"

extern struct player_global_struct player_global;
extern GtkWidget *widget_panel;
extern GtkWidget *widget_playlist;
extern GtkWidget *widget_config;

GtkWidget *panel;
GdkPixmap *panel_bg     = NULL;
GdkGC     *panel_gc;
gint       nPanelPX, nPanelPY;
gint       nPanelWidth, nPanelHeight;
gboolean   bPanel_moving = FALSE ;
GList     *panel_wlist = NULL;

PButton   *panel_buttons[100];


/**
 *
 */

void draw_string (gchar *str_, skin_t *pFont_)
{
	GdkPixmap *pixmap = pFont_->pixmap;
	int nFontWidth, nFontHeight;
	int nCWidth, nCHeight;
	int i, nLen;
	int x_dest, y_dest;
    
	if (!pFont_ || !str_)
		return;

	gdk_window_get_size (pixmap,
		&nFontWidth, &nFontHeight);

	x_dest = pFont_->x;
	y_dest = pFont_->y;

    /*
     * Char width/height
     */

	nCWidth = nFontWidth / 32;
	nCHeight = nFontHeight / 3;
	nLen = strlen (str_);

	for (i=0; i<pFont_->nMax; i++) {
		int c=0;

		if ((i<nLen) && (str_[i] >= 32))
			c = str_[i]-32;

		if (c>=0) {
			int px, py;

			px = (c % 32) * nCWidth;
			py = (c / 32) * nCHeight;

			gdk_draw_pixmap (panel_bg, panel_gc, pixmap,
				px,py, x_dest, y_dest, nCWidth, nCHeight);
		}

		x_dest += nCWidth;
	}
}


/**
 *
 */

void draw_number (gint n_, skin_t *pFont_)
{
    GdkPixmap *pixmap = pFont_->pixmap;
    int nFontWidth, nFontHeight;
    int nCWidth, nCHeight;
    int i;
    int x_dest, y_dest;
    int nDiv = 1;

    if (!pFont_)
	return;

    for (i=0; i<pFont_->nMax-1; i++)
	nDiv*=10;

    gdk_window_get_size (pixmap,
			 &nFontWidth, &nFontHeight);

    x_dest = pFont_->x;
    y_dest = pFont_->y;

    /*
     * Char width/height
     */

    nCWidth = nFontWidth / 11;
    nCHeight = nFontHeight;

    while (nDiv>0)
    {
	int c = n_ / nDiv ;
	int px;

	px = c * nCWidth;

	gdk_draw_pixmap (panel_bg, panel_gc, pixmap,
			 px, 0, x_dest, y_dest, nCWidth, nCHeight);

	x_dest += nCWidth;
	n_ %= nDiv;
	nDiv /= 10;
    }

}

static gboolean inside_sensitive_widgets(gint x, gint y)
{
    GList *wl;
    Widget *w;
    
    wl = panel_wlist;
    while (wl)
    {
	w = (Widget *) wl->data;

//	fprintf (stderr, "Widget : %d\n",w);

	if (inside_widget (x,y,w))
	    return TRUE;

	wl = wl->next;
    }

    return FALSE;
}


/**
 *
 **/
 
void panel_press(GtkWidget * widget, GdkEventButton * event, 
		 gpointer callback_data)
{
//	fprintf (stderr, "panel_press\n");

	if ( (event->button == 1)
		&& (event->type == GDK_BUTTON_PRESS)
		&& !inside_sensitive_widgets(event->x, event->y)) { // move panel
		gdk_window_raise(panel->window);
		gdk_pointer_grab(panel->window, FALSE, GDK_BUTTON_MOTION_MASK 
			| GDK_BUTTON_RELEASE_MASK, GDK_NONE, 
			GDK_NONE, GDK_CURRENT_TIME);

			bPanel_moving = TRUE;

			nPanelPX = event->x;
			nPanelPY = event->y;

	} else {
//		fprintf (stderr, "handle_press\n");
		handle_press_cb(panel_wlist, widget, event);
		draw_panel_window(FALSE);
	}
}


/**
 *
 **/

void panel_release(GtkWidget * widget, GdkEventButton * event, 
		   gpointer callback_data)
{
	gdk_pointer_ungrab(GDK_CURRENT_TIME);
	gdk_flush();

	bPanel_moving = FALSE;

	handle_release_cb (panel_wlist, widget, event);

	draw_panel_window(FALSE);
}


/**
 *
 **/

void panel_motion(GtkWidget * widget, GdkEventMotion * event, 
		  gpointer callback_data)
{
	XEvent ev;
	gint i = 0;
  

	XSync (GDK_DISPLAY(), False);
  
	while (XCheckTypedEvent (GDK_DISPLAY(), MotionNotify, &ev)) {
		event->x = ev.xmotion.x;
		event->y = ev.xmotion.y;
		i++;
	}

	if (bPanel_moving) {
		gint mx, my, newx, newy;
		GdkModifierType modmask;

		gdk_window_get_pointer (NULL, &mx, &my, &modmask);

		newx = mx - nPanelPX;
		newy = my - nPanelPY;
		gdk_window_move(panel->window, newx, newy);

		cfg_set_val_int ("xpos", newx) ;
		cfg_set_val_int ("ypos", newy) ;
	}
  
	gdk_flush();
}


/**
 *
 **/

void panel_set_back_pixmap(void)
{
	gdk_window_set_back_pixmap(panel->window, panel_bg, 0);
	gdk_window_clear(panel->window);  

}


/**
 *
 **/

gint panel_client_event(GtkWidget *w,GdkEventClient *event, gpointer data)
{
	static GdkAtom atom_rcfiles = GDK_NONE;

	if (!atom_rcfiles)
		atom_rcfiles = gdk_atom_intern("_GTK_READ_RCFILES", FALSE);

	if(event->message_type == atom_rcfiles) {
		panel_set_back_pixmap();
		return TRUE;
	}

	return FALSE;
}


/**
 *
 */

void panel_audio_callback (gpointer pData)
{
	omsSetFilter (BUF_AUDIO, (int) pData);
}


/**
 *
 */

void panel_audio_popup (GdkEventButton *pEvent)
{
	GtkWidget *pMenu, *pMenuItem;
	nav_tree_t *audio;
	int i;

    // Generate Menu items
	if (!(audio = omsGetInfo (OMS_INFO_AUDIO)))
		return;

	pMenu = gtk_menu_new ();

	for (i=0; i<audio->num; i++) {
		nav_subtree_t *sub_tree = audio->title+i;

		pMenuItem = gtk_menu_item_new_with_label (sub_tree->name);

		gtk_signal_connect_object (GTK_OBJECT (pMenuItem), "activate",
			GTK_SIGNAL_FUNC (panel_audio_callback),
			(gpointer) i);

		gtk_menu_append (GTK_MENU (pMenu), pMenuItem);
		gtk_widget_show (pMenuItem);
	}
   
//DENT: delete tree on close 

	gtk_menu_popup (GTK_MENU (pMenu), NULL, NULL, NULL, NULL,
		pEvent->button, pEvent->time);
}


/**
 *
 */

void panel_subpicture_callback (gpointer pData)
{
	omsSetFilter (BUF_SUBPIC, (int) pData);
}


/**
 *
 */

void panel_subpicture_popup (GdkEventButton *pEvent)
{
	GtkWidget *pMenu, *pMenuItem;
	nav_tree_t *subpic;
	int i;

    // Generate Menu items
	if (!(subpic = omsGetInfo (OMS_INFO_SUBPIC)))
		return;

	pMenu = gtk_menu_new ();

	for (i=0; i<subpic->num; i++) {
		nav_subtree_t *sub_tree = subpic->title+i;

		pMenuItem = gtk_menu_item_new_with_label (sub_tree->name);

		gtk_signal_connect_object (GTK_OBJECT (pMenuItem), "activate",
			GTK_SIGNAL_FUNC (panel_subpicture_callback),
			(gpointer) i);

		gtk_menu_append (GTK_MENU (pMenu), pMenuItem);
		gtk_widget_show (pMenuItem);
	}
   
//DENT: delete tree on close 

	gtk_menu_popup (GTK_MENU (pMenu), NULL, NULL, NULL, NULL,
		pEvent->button, pEvent->time);
}


/**
 *
 */

void panel_main(int nID_, GdkEventButton *pEvent_)
{
	switch (nID_) {
	case SKIN_CLOSE:	// should only close panel, but for now...
		cfg_save ();
	
		gtk_main_quit();
//		exit(0);
		break;
	case SKIN_EJECT:	// TODO TODO TODO
		break;
	case SKIN_PLAYLIST:	// show/hide playlist
		if (player_global.bPLShowing)
			gtk_widget_hide (widget_playlist);
		else
			gtk_widget_show (widget_playlist);
	
		player_global.bPLShowing = !player_global.bPLShowing;
		break;
	case SKIN_CONFIG:	// show/hide config dialog
		if (player_global.bConfShowing)
			gtk_widget_hide (widget_config);
		else
			gtk_widget_show (widget_config);
	
		player_global.bConfShowing = !player_global.bConfShowing;
		break;
	case SKIN_PREV:
		playlist_play_prev();
		break;
	case SKIN_PAUSE: 
		switch (omsGetStatus ()) {
		case STATUS_FINISH:
		case STATUS_STOP:
		case STATUS_PAUSE_ON:
			break;

		case STATUS_PAUSE_OFF:
		case STATUS_PLAY:
			omsSetStatus (STATUS_PAUSE_ON);
			panel_update_ui();
			break;
		}
		break;
	case SKIN_PLAY: {
		switch (omsGetStatus ()) {
		case STATUS_FINISH:
		case STATUS_STOP:
			playlist_play_current();
			break;
		case STATUS_PAUSE_ON:
			omsSetStatus (STATUS_PAUSE_OFF);
			panel_update_ui();
			break;
		}
		break;
	}
	case SKIN_STOP:
		if (omsGetStatus () != STATUS_STOP) {
			omsSetStatus (STATUS_STOP);
			panel_update_ui();
		}
		break;
	case SKIN_NEXT:
		playlist_play_next();
		break;
	case SKIN_AUDIO:
		panel_audio_popup (pEvent_);
		break;
	case SKIN_SUBPIC:
		panel_subpicture_popup (pEvent_);
		break;
	}
}


/**
 *
 */

void panel_update_ui () 
{
	int nStatus = omsGetStatus();

	switch (nStatus) {
	case STATUS_STOP:
		pbutton_set_status (panel_buttons[SKIN_STOP], TRUE);
		pbutton_set_status (panel_buttons[SKIN_PLAY], FALSE);
		pbutton_set_status (panel_buttons[SKIN_PAUSE],FALSE);
		break;
	case STATUS_PAUSE:
		pbutton_set_status (panel_buttons[SKIN_STOP], FALSE);
		pbutton_set_status (panel_buttons[SKIN_PLAY], FALSE);
		pbutton_set_status (panel_buttons[SKIN_PAUSE],TRUE);
		break;
	case STATUS_PLAY:
		pbutton_set_status (panel_buttons[SKIN_STOP], FALSE);
		pbutton_set_status (panel_buttons[SKIN_PLAY], TRUE);
		pbutton_set_status (panel_buttons[SKIN_PAUSE],FALSE);
		break;
	default:
		fprintf (stderr, "Strange status : %d\n", nStatus);
	}

	draw_panel_window (TRUE);
}


/**
 *
 **/

void draw_panel_window(gboolean force)
{
    gboolean redraw;

    lock_widget_list(panel_wlist);

// Main Skin
    gdk_draw_pixmap(panel_bg,  panel_gc, get_skin_pixmap (SKIN_BACKGROUND), 
		    0, 0, 0, 0, nPanelWidth, nPanelHeight);
  
    // =>Playlist  GetCurrentLocation (&title, &part, &timecode);

    draw_string (player_global.strCurSourceName, get_skin (SKIN_SOURCE));
    draw_number (player_global.nCurChapter, get_skin (SKIN_CNUM));
   
    draw_string (player_global.strCurChapter, get_skin (SKIN_CHAPTER));

    draw_number (player_global.nCurTitle+1, get_skin (SKIN_TNUM));


// draw widgets
    draw_widget_list (panel_wlist, &redraw, TRUE);
    gdk_window_clear (panel->window);
    gdk_flush ();

    unlock_widget_list (panel_wlist);
}


/**
 *
 */

void apply_panel_skin ()
{
    skin_t *maskskin;
    int i;

/* GB: Sorry for many memory leaks here but this wont be called too often */
// DENT: no problem right now - we've still plenty of leaks hanging around, 
//	but we've to fix them step by step

    gdk_window_get_size (get_skin_pixmap (SKIN_BACKGROUND),
			 &nPanelWidth, &nPanelHeight);

    /* fprintf (stderr, "size : %d x %d\n",nPanelWidth, nPanelHeight); */

    gtk_widget_set_usize(GTK_WIDGET(panel), nPanelWidth, nPanelHeight);

    /*
     * set back pixmap
     */

    panel_bg = gdk_pixmap_new(panel->window, nPanelWidth, nPanelHeight, 
			      gdk_visual_get_best_depth());

    /* fprintf (stderr, "BackPixmap : %d x %d\n",nPanelWidth, nPanelHeight); */

    gdk_window_set_back_pixmap(panel->window, panel_bg, 0);

    /*
     * shape window
     */

    /* GB: Doesnt work properly */

    fprintf (stderr, "shape window\n");
    gtk_widget_shape_combine_mask (panel, NULL, 0, 0);

    if ((maskskin = get_skin (SKIN_MASK))) {
	gtk_widget_shape_combine_mask (panel, maskskin->mask, 0, 0);
    } else {
	if ((maskskin = get_skin (SKIN_BACKGROUND)))
	    gtk_widget_shape_combine_mask (panel, maskskin->mask, 0, 0);
    }


    /*
     * setup buttons:
     */

    
	for (i=SKIN_PLAY; i<=SKIN_CLOSE; i++) {
		skin_t *pSkin = get_skin (i);

		if (pSkin) {
			int nWidth, nHeight;

			if (!(pSkin->pixmap))
				continue;

			gdk_window_get_size (pSkin->pixmap,
				&nWidth, &nHeight);

			if (pSkin->bPrelight) {
				nWidth /= pSkin->bStatus ? 6 : 3;
			} else
				nWidth /= 2;

			change_pbutton(panel_buttons[i],
				panel_bg,
				pSkin->x, pSkin->y,
				nWidth, nHeight,
				pSkin->pixmap,
				pSkin->bPrelight,
				pSkin->bStatus,
				i);
		}
	}

	draw_panel_window (TRUE);
}


/**
 *
 **/

GtkWidget *setup_panel () 
{
	skin_t *maskskin;
	int i;

	gdk_threads_enter();

//	fprintf (stderr, "%s/%d\n", __FILE__, __LINE__);
	gdk_window_get_size (get_skin_pixmap (SKIN_BACKGROUND),
		&nPanelWidth, &nPanelHeight);

	panel = gtk_window_new (GTK_WINDOW_TOPLEVEL);

	gtk_widget_set_app_paintable(panel, TRUE);
	gtk_window_set_title(GTK_WINDOW(panel), "oms - Open Media System");
	gtk_window_set_policy(GTK_WINDOW(panel), FALSE, FALSE, TRUE);

	gtk_widget_set_usize(GTK_WIDGET(panel), nPanelWidth, nPanelHeight);

	gtk_signal_connect (GTK_OBJECT(panel), "destroy",
		GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

	gtk_widget_set_events(panel, GDK_FOCUS_CHANGE_MASK 
		| GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK 
		| GDK_BUTTON_RELEASE_MASK | GDK_STRUCTURE_MASK);

	gtk_widget_realize(panel);

	gdk_window_set_decorations(panel->window, 0);

	panel_gc = gdk_gc_new(panel->window);

    /*
     * set back pixmap
     */

	panel_bg = gdk_pixmap_new(panel->window, nPanelWidth, nPanelHeight, 
		gdk_visual_get_best_depth());

	gdk_window_set_back_pixmap(panel->window, panel_bg, 0);

    /*
     * shape window
     */

	if ((maskskin = get_skin (SKIN_MASK))) {
		gtk_widget_shape_combine_mask (panel, maskskin->mask, 0, 0);
	} else if ((maskskin = get_skin (SKIN_BACKGROUND))) 
			gtk_widget_shape_combine_mask (panel, maskskin->mask, 0, 0);
	
	gtk_signal_connect(GTK_OBJECT(panel), "button_press_event", 
		GTK_SIGNAL_FUNC(panel_press), NULL);
	gtk_signal_connect(GTK_OBJECT(panel), "button_release_event", 
		GTK_SIGNAL_FUNC(panel_release), NULL);
	gtk_signal_connect(GTK_OBJECT(panel), "motion_notify_event", 
		GTK_SIGNAL_FUNC(panel_motion), NULL);
	gtk_signal_connect(GTK_OBJECT(panel), "client_event",
		GTK_SIGNAL_FUNC(panel_client_event), NULL);

	gdk_window_move (panel->window, 200, 0);

    /*
     * generate buttons:
     */

	for (i=SKIN_PLAY; i<=SKIN_CLOSE; i++) {
		skin_t *pSkin = get_skin (i);
	
		if (pSkin && pSkin->pixmap) {
			int nWidth, nHeight;

			gdk_window_get_size (pSkin->pixmap, &nWidth, &nHeight);

			if (pSkin->bPrelight) {
				if (pSkin->bStatus)
					nWidth /= 6;
				else
					nWidth /= 3;
			} else
				nWidth /= 2;

			panel_buttons[i] = create_pbutton (&panel_wlist,
						panel_bg, 
						panel_gc,
						pSkin->x, pSkin->y,
						nWidth, nHeight,
						pSkin->pixmap,
						pSkin->bPrelight,
						pSkin->bStatus,
						panel_main, i);

			if ((i==SKIN_AUDIO) || (i==SKIN_SUBPIC))
				pbutton_set_immediate (panel_buttons[i], TRUE);
		}
	}

	gdk_threads_leave();

	return panel;
}
