/*  Open Media System - oms
 *
 * Copyright (C) 1999  Thomas Mirlacher Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include "gtk/gtk.h"
#include "pbutton.h"
#include "skin.h"

extern GdkPixmap * get_skin_pixmap(int index_);

/**
 *
 */

void pbutton_draw(Widget *w)
{
	PButton *button = (PButton *) w;
	GdkPixmap *src;
	GdkPixmap *obj;
	int x_offset;
 
	if (!button)
		return;

	if (!button->mbAllowDraw)
		return;
    
	obj = button->mWidget.parent;
	if (!(src = button->mpSkin))	// dont draw a non-existing pixmap
		return;
    
    /*
     * find out which part of the skin (=>which status of the button) to display
     */
    
	if (button->mbInside) {
		if (button->mbPressed) {
			if (button->mbHasStatus && button->mbStatus)
				x_offset = button->mWidget.width * 3;
			else
				x_offset = button->mWidget.width;
		} else {
			if (button->mbHasStatus) {
				if (button->mbStatus)
					x_offset = button->mWidget.width * 5;
				else
					x_offset = button->mWidget.width * 3;
			} else {
				if (button->mbPrelight)
					x_offset = button->mWidget.width * 2;
				else
					x_offset = 0;
			}
		}
	} else {
		if (button->mbStatus && button->mbHasStatus)
			x_offset = button->mWidget.width * 2;
		else
			x_offset = 0;
	}
    
	gdk_draw_pixmap(obj, button->mWidget.gc,
		src, x_offset, 0,
		button->mWidget.x, button->mWidget.y, 
		button->mWidget.width, button->mWidget.height);
}

void pbutton_button_press_cb(GtkWidget * widget, 
			     GdkEventButton * event, PButton * button)
{
  if (event->button != 1)
    return;
  if (inside_widget(event->x, event->y, &button->mWidget))
  {
    button->mbPressed = 1;
    button->mbInside = 1;
    draw_widget(button);
    if ( (button->mPushCb) && (button->mbImmediate))
      button->mPushCb(button->mnId, event);
  }
}

void pbutton_button_release_cb(GtkWidget * widget, 
			       GdkEventButton * event, PButton * button)
{
  if (event->button != 1)
    return;
  if (button->mbInside && button->mbPressed)
  {
    button->mbInside = 0;
    draw_widget(button);
    if ( (button->mPushCb) && (!button->mbImmediate))
      button->mPushCb(button->mnId, event);
  }
  if (button->mbPressed)
    button->mbPressed = 0;
}

void pbutton_motion_cb(GtkWidget * widget, 
		       GdkEventMotion * event, PButton * button)
{
  int bInside;

  if (!button->mbPressed)
    return;

  bInside = inside_widget(event->x, event->y, &button->mWidget);
  if (bInside != button->mbInside)
  {
    button->mbInside = bInside;
    draw_widget(button);
  }
}

void pbutton_set_status(PButton *b, gboolean bStatus)
{
  b->mbStatus = bStatus;
  draw_widget (b);
}

void pbutton_set_immediate (PButton *b, gboolean bImmediate)
{
  b->mbImmediate = bImmediate;
  draw_widget (b);
}

PButton *create_pbutton(GList ** wlist, GdkPixmap * parent, 
			GdkGC * gc, 
			gint x, gint y, gint w, gint h,
			GdkPixmap *pSkin,
			gboolean   bPrelight,
			gboolean   bHasStatus,
			void (*cb) (int nID, GdkEventButton *pEvent), gint nID)
{
	PButton *b;

	b = (PButton *) g_malloc0(sizeof (PButton));
	b->mWidget.parent = parent;
	b->mWidget.gc = gc;
	b->mWidget.x = x;
	b->mWidget.y = y;
	b->mWidget.width = w;
	b->mWidget.height = h;
	b->mWidget.visible = 1;
	b->mWidget.button_press_cb = GTK_SIGNAL_FUNC(pbutton_button_press_cb);
	b->mWidget.button_release_cb = GTK_SIGNAL_FUNC(pbutton_button_release_cb);
	b->mWidget.motion_cb = GTK_SIGNAL_FUNC(pbutton_motion_cb);
	b->mWidget.draw = pbutton_draw;
	b->mPushCb = cb;
	b->mpSkin   = pSkin;
	b->mbPrelight  = bPrelight;
	b->mbHasStatus = bHasStatus;
	b->mbAllowDraw = TRUE;
	b->mbImmediate = FALSE;
	b->mnId = nID;
	add_widget(wlist, b);

	return b;
}

void change_pbutton(PButton *b, 
		    GdkPixmap *parent,
		    gint x, gint y, gint w, gint h,
		    GdkPixmap *pSkin,
		    gboolean   bPrelight,
		    gboolean   bHasStatus,
		    gint nID)
{
  b->mWidget.parent = parent;
  b->mWidget.x = x;
  b->mWidget.y = y;
  b->mWidget.width = w;
  b->mWidget.height = h;
  b->mpSkin   = pSkin;
  b->mbPrelight  = bPrelight;
  b->mbHasStatus = bHasStatus;
  b->mnId = nID;
}

void free_pbutton(PButton * b)
{
  g_free(b);
}
