/* Open Media System - oms
 *
 * the playlist
 *
 * Copyright (C) 1999, 2000  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/*
 * DENT: split this into 3 different files
 *	1) playlist general stuff
 *	2) playlist flatmode related
 *	3) playlist treemode functions
 */

#include "playlist.h"

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <gdk/gdktypes.h>

#include "main.h"
#include "config_devices.h"
#include "oms/oms.h"
#include "panel_gui.h"
#include "messagebox.h"

void cb_chapter_change (void *data, int my_part);
void cb_itemsignal (GtkWidget *item, gchar *signame);

struct omsitem_info
{
    gchar *strInput;
    gchar *strTitle; /* of DVD etc. */
    gint   nTitle;
    gint   nChapter;
    gchar *strChapter;
};

void scroll_init (void);
extern struct player_global_struct player_global;
extern GtkWidget *widget_playlist;

/*
 * Global variables
 */

struct { 
    GtkWidget           *flatlist;
    GtkTree             *tree;
    GtkScrolledWindow   *flat_scroller;
    GtkWidget           *pFileTreeItem;
    GtkWidget           *pFileTree;
    
    gboolean             bFirstSet; /* TRUE if initial setup of  */
    /* player_global is done */
    struct omsitem_info *pCurNavItem;
    gboolean             bFlatlistMode;
    gint                 nSelectedRow; /* in flatlist */
    gboolean             bSelectionInProgress;
} playlist_global = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    FALSE,
    NULL,
    TRUE,
    0,
    FALSE
};

/*
 *
 */

void playlist_play_item (struct omsitem_info *pItem)
{
	if (!pItem) {
		omsSetStatus (STATUS_STOP);
		panel_update_ui ();
	
		return;
	}

    /*
     * switch input source ? 
     */
 
    if (!player_global.strCurSource 
	|| strcmp (player_global.strCurSource, pItem->strInput)) {
#ifdef DEBUG
	fprintf (stderr, "SWITCHING INPUT FROM %s to %s \n", player_global.strCurSource, pItem->strInput);
#endif

	if (omsOpenInput (pItem->strInput) < 0) {
	    char errstr[100];
	    sprintf (errstr, "Error switching input to %s", pItem->strInput);
	    show_message ("Error", errstr, MB_OK);
	    return;
	}

	/*
	  omsUnregisterCb (CB_CHANGE);
	  omsRegisterCb (CB_CHANGE, cb_chapter_change, NULL);
	*/

	if (player_global.strCurSource)
	    free (player_global.strCurSource);
	player_global.strCurSource = g_strdup(pItem->strInput);

	if (player_global.strCurSourceName)
	    free (player_global.strCurSourceName);
	player_global.strCurSourceName = g_strdup(pItem->strTitle);
    }

    PartPlay (pItem->nTitle, pItem->nChapter);

    playlist_global.pCurNavItem = pItem;

    player_global.nCurTitle   = pItem->nTitle;
    player_global.nCurChapter = pItem->nChapter;
    player_global.strCurSourceName = g_strdup(pItem->strTitle);

    if (player_global.strCurChapter)
	free (player_global.strCurChapter);
    player_global.strCurChapter = g_strdup (pItem->strChapter);

    panel_update_ui();
}


/**
 * DnD stuff
 */

enum {
    TARGET_STRING,
    TARGET_ROOTWIN
};


/**
 *
 */

static GtkTargetEntry target_table[] = {
    { "STRING",     0, TARGET_STRING },
    { "text/plain", 0, TARGET_STRING }
};

static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);


/**
 *
 */

static void _add_flatlist_entry (gchar *pathname, gint nRow, gchar *Input)
{
	struct omsitem_info *pNavItemInfo;
	GtkWidget           *pItem;
	gchar               *texts[2];
	gchar               *ptr;

	if ((ptr = strchr (pathname, ':'))) {
		texts[0] = pathname;
		*ptr = '\0';
		pathname = ptr+1;
	} else
		texts[0] = '\0';

	texts[1] = pathname;

	nRow = nRow>0 ?  nRow-1 : 0;
      
    /*
     * generate omsitem for file
     */
  
	pNavItemInfo = (struct omsitem_info*) malloc (sizeof (struct omsitem_info));
	pNavItemInfo->strInput  = g_strdup (Input ? Input : texts[1]);
	pNavItemInfo->strTitle  = pNavItemInfo->strInput;
	pNavItemInfo->nTitle    = 0;
	pNavItemInfo->nChapter  = 0;
	pNavItemInfo->strChapter= g_strdup (texts[0]);
  
    /*
     * add to flatlist
     */

	gtk_clist_insert( (GtkCList*) playlist_global.flatlist, nRow, texts);
	gtk_clist_set_row_data ( (GtkCList *) playlist_global.flatlist, nRow, pNavItemInfo);    
	texts[1] = g_strdup(pathname);

    /*
     * add to tree
     */

	pItem = gtk_tree_item_new_with_label (g_strdup(pathname));
	gtk_object_set_data(GTK_OBJECT(pItem), "item_info", pNavItemInfo); 

	gtk_tree_append (GTK_TREE(playlist_global.pFileTree), pItem);
    
	gtk_signal_connect (GTK_OBJECT(pItem), "select",
		GTK_SIGNAL_FUNC(cb_itemsignal), "select");
    
	gtk_widget_show (pItem);
}


/**
 * Drag'n'Drop received
 */

void flatlist_drag_data_received  (GtkWidget          *widget,
				   GdkDragContext     *context,
				   gint                x,
				   gint                y,
				   GtkSelectionData   *data,
				   guint               info,
				   guint               time)
{
	int nRow=0, nCol=0;
	gchar *string = data->data;
	GtkAdjustment* pAdj = gtk_scrolled_window_get_vadjustment (playlist_global.flat_scroller);

	if ((data->length < 0) || (data->format != 8))
		return;

	gtk_clist_get_selection_info ((GtkCList *) widget,
		x, y+pAdj->value, &nRow, &nCol);

	while (string && *string) {
		gchar *next_string = NULL;

		if ((next_string = strchr (string, '\n'))) {
			if (*(next_string - 1) == '\r')
				*(next_string - 1) = '\0';
			else
				*next_string = '\0';

			next_string++;
		}

#ifdef DEBUG
//		fprintf (stderr,"DROPPED: %s at %d \n", string, nRow);
#endif
		_add_flatlist_entry (string, nRow, NULL);
	  
		gtk_drag_finish (context, TRUE, FALSE, time);
		nRow++;

		string = next_string;
	}
  
	gtk_drag_finish (context, FALSE, FALSE, time);
}


/**
 *
 */

void hide_playlist () 
{
    gtk_widget_hide (widget_playlist);
    player_global.bPLShowing = FALSE;
}


/**
 *
 */

#define NAVITEM_CURRENT 0
#define NAVITEM_NEXT    1
#define NAVITEM_PREV    2

static struct omsitem_info *playlist_get_omsitem_flatlist (int nWhich)
{
    gint nRow, nDesired=0;

    /*fprintf (stderr, "flatlist mode\n");*/

    nRow = gtk_clist_find_row_from_data  ((GtkCList *) playlist_global.flatlist,
					  playlist_global.pCurNavItem);

    switch (nWhich) {
    case NAVITEM_CURRENT:
	nDesired = nRow;
	break;
    case NAVITEM_NEXT:
	nDesired = nRow+1;
	break;
    case NAVITEM_PREV:
	nDesired = nRow-1;
	break;
    }

    /* fprintf (stderr, "ndesired %d\n", nDesired); */

    if (nDesired >= 0) {
	struct omsitem_info* pNavItemInfo;
	  
	pNavItemInfo = (struct omsitem_info *) gtk_clist_get_row_data ((GtkCList *) playlist_global.flatlist, nDesired);
      
	if (pNavItemInfo) {
	    playlist_global.bSelectionInProgress = TRUE;

	    gtk_clist_unselect_row ((GtkCList *) playlist_global.flatlist,
				    nRow, 0);

	    gtk_clist_select_row ((GtkCList *) playlist_global.flatlist,
				  nDesired, 0);

	    playlist_global.bSelectionInProgress = FALSE;
	}  
	return pNavItemInfo;
    }

    return NULL;
}


/**
 *
 */

static struct omsitem_info *playlist_get_omsitem_treelist (int nWhich)
{
    GList *pChildren;
    /*
     * search device
     */

    pChildren =  playlist_global.tree->children;

    while (pChildren) {
	gchar *str;

	GtkTreeItem *pDeviceItem = (GtkTreeItem *) pChildren->data;
	str = (char*) gtk_object_get_data(GTK_OBJECT(pDeviceItem), "device name");
	  
	/* fprintf (stderr, "DEVICE: %s =? %s\n",str,playlist_global.pCurNavItem->strInput); */

	if (!strcmp (str, playlist_global.pCurNavItem->strInput)) {
	    /*
	     * get title
	     */

	    GtkTree *pTitleTree = (GtkTree *)pDeviceItem->subtree;
	    GtkTreeItem *pTitleItem = (GtkTreeItem *) g_list_nth(pTitleTree->children, playlist_global.pCurNavItem->nTitle)->data;

	    if (pTitleItem) {
		GtkTree *pChapterTree = (GtkTree *)pTitleItem->subtree;
		GtkTreeItem *pChapterItem;
		int nDesired=0;

		switch (nWhich) {
		case NAVITEM_CURRENT:
		    nDesired = playlist_global.pCurNavItem->nChapter;
		    break;
		case NAVITEM_NEXT:
		    nDesired = playlist_global.pCurNavItem->nChapter+1;
		    break;
		case NAVITEM_PREV:
		    nDesired = playlist_global.pCurNavItem->nChapter-1;
		    break;
		}

		/* fprintf (stderr, "Desired : %d\n",nDesired); */
	  
		if (nDesired<0)
		    return NULL;

		pChapterItem = (GtkTreeItem *) g_list_nth(pChapterTree->children, nDesired)->data;
		if (pChapterItem) {
		    //					fprintf (stderr, "got Chapter item! :)))\n");
		    gtk_tree_select_child (pChapterTree, (GtkWidget *) pChapterItem);
		    return (struct omsitem_info*) gtk_object_get_data(GTK_OBJECT(pChapterItem),"item_info");
		}
	    } else
		fprintf (stderr, "TREE: TitleItem not found!!\n");
	}
	pChildren = g_list_next (pChildren);
    }
    /*	fprintf (stderr, "Tree mode\n"); */
    return NULL;
}


/**
 *
 */

struct omsitem_info *playlist_get_omsitem (int nWhich)
{
    if (!playlist_global.pCurNavItem)
	return NULL;

    if (playlist_global.bFlatlistMode) {
	return playlist_get_omsitem_flatlist (nWhich);
    } else {	/* Tree Mode */
		       return playlist_get_omsitem_treelist (nWhich);
    }
}


/**
 *
 */

void playlist_play_next (int when)
{
    switch (when) {
    case 0:
	fprintf (stderr, "playing next NOW!\n");
	break;
    case 1:
	fprintf (stderr, "playing next LATER!\n");
	break;
    }

    playlist_play_item (playlist_get_omsitem (NAVITEM_NEXT));

}


/**
 *
 */

void playlist_play_prev ()
{
    playlist_play_item (playlist_get_omsitem (NAVITEM_PREV));
}


/**
 *
 */

void playlist_play_current () 
{
    playlist_play_item (playlist_global.pCurNavItem);
}


/**
 *
 */

void playlist_selection_made(GtkWidget *widget,
			     gint row,
			     gint column,
			     GdkEventButton *event,
			     gpointer data)
{
    struct omsitem_info *pNavItemInfo;

    if (playlist_global.bSelectionInProgress) 
	return;

    playlist_global.bSelectionInProgress = TRUE;

    playlist_global.nSelectedRow = row;
    playlist_global.bFlatlistMode = TRUE;

    if ((event->type==GDK_2BUTTON_PRESS)
	||(event->type==GDK_3BUTTON_PRESS) ) {
	pNavItemInfo = (struct omsitem_info *) gtk_clist_get_row_data ((GtkCList *) playlist_global.flatlist, row);
	playlist_play_item (pNavItemInfo);
    }

    playlist_global.bSelectionInProgress = FALSE;
}


/**
 * tree selection
 */

void cb_itemsignal (GtkWidget *item, gchar *signame)
{
    struct omsitem_info *pNavItemInfo;

    pNavItemInfo = (struct omsitem_info*) gtk_object_get_data(GTK_OBJECT(item), "item_info");

    playlist_global.bFlatlistMode = FALSE;
    playlist_play_item (pNavItemInfo);
}


/**
 *
 */

void cb_chapter_change (void *data, int my_part)
{
    struct omsitem_info *pNavItemInfo;

    fprintf (stderr,"\n\n CALLBACK CALLED \n\n");

    playlist_play_next (1);

    if ((pNavItemInfo = playlist_get_omsitem (NAVITEM_NEXT))) {
	THREADSAFE(playlist_play_item (pNavItemInfo));
    } else if (omsGetStatus () != STATUS_STOP) {
	omsSetStatus (STATUS_STOP);
	panel_update_ui();
    }
}


/**
 *
 */

void playlist_move_up (GtkButton *button, gpointer user_data) 
{
    if (playlist_global.nSelectedRow>0) {
	gtk_clist_row_move ((GtkCList *) playlist_global.flatlist,
			    playlist_global.nSelectedRow, playlist_global.nSelectedRow-1);
	playlist_global.nSelectedRow--;
    }
}


/**
 *
 */

void playlist_move_down (GtkButton *button, gpointer user_data) 
{
    GtkCList *pList = (GtkCList *) playlist_global.flatlist;

    if (playlist_global.nSelectedRow<(pList->rows-1)) {
	gtk_clist_row_move ((GtkCList *) playlist_global.flatlist,
			    playlist_global.nSelectedRow, playlist_global.nSelectedRow+1);
	playlist_global.nSelectedRow++;
    }
}


/**
 *
 */

void playlist_remove (GtkButton *button,
		      gpointer user_data) 
{
    struct omsitem_info* pNavItemInfo;
    GtkCList *pList = (GtkCList *) playlist_global.flatlist;

    if (playlist_global.nSelectedRow>=0) {
	pNavItemInfo = (struct omsitem_info *) gtk_clist_get_row_data (pList, playlist_global.nSelectedRow);

	if (pNavItemInfo == playlist_global.pCurNavItem)
	    playlist_play_next (0);

	gtk_clist_remove (pList, playlist_global.nSelectedRow);

	/* select same row (which is the next item now) if possible */

	if (playlist_global.nSelectedRow<pList->rows) {
	    playlist_global.bSelectionInProgress = TRUE;
	    
	    gtk_clist_select_row ((GtkCList *) playlist_global.flatlist,
				  playlist_global.nSelectedRow, 0);
	    
	    playlist_global.bSelectionInProgress = FALSE;
	}
    }
}


/**
 *
 */

void playlist_file_ok_sel (GtkWidget *w, GtkFileSelection *fs)
{
    _add_flatlist_entry (gtk_file_selection_get_filename (fs), 
			playlist_global.nSelectedRow,
			NULL); 
    gtk_widget_destroy (GTK_WIDGET(fs));
}


/**
 *
 */

void playlist_add_file (GtkButton *button,
			gpointer user_data) 
{
    GtkWidget *pFileSel ;

    pFileSel = gtk_file_selection_new ("File selection");

    /*
      gtk_signal_connect (GTK_OBJECT (pFileSel), "destroy",
      (GtkSignalFunc) playlist_filesel_destroy, &pFileSel);
    */

    /* Connect the ok_button to file_ok_sel function */
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (pFileSel)->ok_button),
			"clicked", (GtkSignalFunc) playlist_file_ok_sel, 
			pFileSel );
         
    /* Connect the cancel_button to destroy the widget */
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					   (pFileSel)->cancel_button),
			       "clicked", (GtkSignalFunc) gtk_widget_destroy,
			       GTK_OBJECT (pFileSel));
         
    /*
      gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), 
      "penguin.png");
    */   
    gtk_widget_show(pFileSel);
}


/**
 *
 */

void playlist_clear (GtkButton *button,
		     gpointer user_data) 
{
    if (playlist_global.bFlatlistMode) {
	playlist_global.pCurNavItem = NULL;
	player_global.strCurSourceName = "none";
	player_global.strCurChapter    = "";
	player_global.nCurChapter = 0;
	player_global.nCurTitle = 0;

	if (omsGetStatus () != STATUS_STOP) {
	    omsSetStatus (STATUS_STOP);
	    panel_update_ui();
	}
    }

    gtk_clist_clear ((GtkCList*) playlist_global.flatlist);
}


/**
 *
 */

GtkTree *playlist_scan_device (GtkButton *button, gchar *strDevName)
{
    GtkTree             *pSubTree, *pTitleTree=NULL;
    GtkWidget           *pItem;
    gint                 i, s, npos=0;
    nav_tree_t          *nav_tree;
    gchar               *strlist[3] = { NULL, NULL, NULL } ;
    gchar               *str;
    struct omsitem_info *pNavItemInfo;
    GtkCList            *pList = (GtkCList *) playlist_global.flatlist;
    GList               *pChildren;

    gtk_widget_hide (GTK_WIDGET(playlist_global.tree));
    gtk_widget_hide (playlist_global.flatlist);

    /*
     * clear all entries of this device
     */

    /* clear playlist */
    fprintf (stderr, "Clear playlist \n");

    for (i=0; i<pList->rows; ) {
	pNavItemInfo = (struct omsitem_info *) gtk_clist_get_row_data (pList,i); 

	if (strcmp(pNavItemInfo->strInput, strDevName)==0)
	    gtk_clist_remove(pList,i);
	else
	    i++;
    }

    /* clear tree */
    pChildren =  playlist_global.tree->children;
    
    while (pChildren) {
	gchar *str;
	
	GtkTreeItem *pDeviceItem = (GtkTreeItem *) pChildren->data;
	str = (char*) gtk_object_get_data(GTK_OBJECT(pDeviceItem), "device name");
	
	if ((str) && (strcmp (str, strDevName) == 0)) { /* found it! */
	    if (GTK_TREE(pDeviceItem->subtree)) {
		gtk_tree_item_remove_subtree(GTK_TREE_ITEM(pDeviceItem));

		pTitleTree = (GtkTree *) gtk_tree_new();
		gtk_tree_item_set_subtree (GTK_TREE_ITEM(pDeviceItem), GTK_WIDGET(pTitleTree));
	    }
	}
	pChildren = g_list_next (pChildren);
    }


    /*
     * DENT: free the nav_tree !!!
     *
     */
    omsCloseInput ();

    if (omsOpenInput (strDevName) < 0) { 
	gchar errstr[100];
	sprintf (errstr, "Error opening device\n%s\nfor scanning", strDevName);
	THREADSAFE (show_message ("Error",errstr,MB_OK));
      
	pTitleTree = GTK_TREE(gtk_tree_new()); 
	pItem = gtk_tree_item_new_with_label ("error"); 
	gtk_tree_append (pTitleTree, pItem); 
	gtk_widget_show (pItem); 
	gtk_widget_show (GTK_WIDGET(playlist_global.tree));
	gtk_widget_show (playlist_global.flatlist);

	return pTitleTree; 
    }  


    nav_tree = omsGetInfo (OMS_INFO_PROGRAM);
  
    if (!pTitleTree)
	pTitleTree = (GtkTree *) gtk_tree_new();

    for (i=0; i<nav_tree->num; i++) {
	GtkWidget *item;
	nav_subtree_t *nav_sub = nav_tree->title+i;
    
	pItem = gtk_tree_item_new_with_label (g_strdup(nav_sub->name));
	gtk_tree_append (GTK_TREE(pTitleTree), pItem);
	gtk_widget_show (pItem);

	pSubTree = (GtkTree*) gtk_tree_new();
	gtk_tree_item_set_subtree (GTK_TREE_ITEM(pItem), GTK_WIDGET(pSubTree));
    
	for (s=0; s<nav_sub->num; s++) {
	    nav_tree_data_t *oms_data = nav_sub->sub+s;

	    pNavItemInfo = (struct omsitem_info*) malloc (sizeof (struct omsitem_info));
	    pNavItemInfo->strInput   = strDevName;
	    pNavItemInfo->strTitle   = nav_sub->name;
	    pNavItemInfo->nTitle     = i;
	    pNavItemInfo->nChapter   = s;
	    pNavItemInfo->strChapter = g_strdup(oms_data->name);

	    if (!playlist_global.bFirstSet) {
		player_global.nCurTitle     = i;
		player_global.nCurChapter   = s;
		player_global.strCurSourceName = g_strdup (nav_tree->title->name);
		player_global.strCurSource  = g_strdup (strDevName);
		player_global.strCurChapter = g_strdup(oms_data->name);
		panel_update_ui();
		playlist_global.pCurNavItem = pNavItemInfo;
		playlist_global.bFirstSet = TRUE;
	    }
      
	    item = gtk_tree_item_new_with_label (g_strdup(oms_data->name));
	    gtk_object_set_data(GTK_OBJECT(item), "item_info", pNavItemInfo); 


	    gtk_tree_append (GTK_TREE(pSubTree), item);
      
	    gtk_signal_connect (GTK_OBJECT(item), "select",
				GTK_SIGNAL_FUNC(cb_itemsignal), "select");
      
	    gtk_widget_show (item);

	    /*
	     * playlist entry
	     */

	    str = malloc (10);
	    sprintf (str, "%d", i+1);

	    strlist[0] = str;
	    strlist[1] = g_strdup(oms_data->name);
	    npos = gtk_clist_append( (GtkCList *) playlist_global.flatlist, (gchar **) &strlist);
	    gtk_clist_set_row_data ( (GtkCList *) playlist_global.flatlist, npos, pNavItemInfo);

		free (strlist[1]);
	    free (str);
	}
    }

    gtk_widget_show (GTK_WIDGET(playlist_global.tree));
    gtk_widget_show (playlist_global.flatlist);

    return pTitleTree;
}


/**
 * go through all devices an collect playable items from them
 */

void scan_devices ()
{
	GtkWidget  *pItem;
	GList      *pDevList, *pDevCur;
	gchar      *strDevName;

	playlist_global.bFirstSet = FALSE;

    /*
     * TODO: clear Tree
     */

	pDevList = get_dev_list () ; /* from config_devices */

    /* 
     * now : scan devices
     */
    
	pDevCur = g_list_first (pDevList);

	while (pDevCur) {
		strDevName = (gchar *) pDevCur->data ;

		pItem = gtk_tree_item_new_with_label (strDevName);
		gtk_object_set_data(GTK_OBJECT(pItem), "device name", g_strdup(strDevName)); 
		gtk_tree_append (GTK_TREE(playlist_global.tree), pItem);
		gtk_widget_show (pItem);
    
		gtk_tree_item_set_subtree (GTK_TREE_ITEM(pItem), 
			GTK_WIDGET(playlist_scan_device (NULL, strDevName)));

		gtk_tree_item_expand (GTK_TREE_ITEM(pItem));

		pDevCur = g_list_next (pDevCur);
	}  
}


/**
 *
 */

GtkWidget* setup_playlist ()
{
    GtkWidget *win, *scrolled_win;
    GtkWidget *notebook, *label, *vbox, *hbox, *button;
    GList     *pDevList, *pDevCur;
    gchar     *strTitles[3] = { "Source", "Caption", NULL } ;
  
    win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_signal_connect (GTK_OBJECT(win), "delete_event",
			GTK_SIGNAL_FUNC (hide_playlist), NULL);
  
    gtk_window_set_title(GTK_WINDOW(win), "DVD Navigator");
  

    /*
     * notebook for flatflist and tree 
     */

    notebook = gtk_notebook_new ();
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
    gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
    gtk_container_add (GTK_CONTAINER(win), notebook);
    gtk_widget_show(notebook);

    /*
     * the flatlist ("playlist view") 
     */

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_widget_show (hbox);

    playlist_global.flat_scroller = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (playlist_global.flat_scroller),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize (GTK_WIDGET(playlist_global.flat_scroller), 320, 200);
    gtk_widget_show (GTK_WIDGET(playlist_global.flat_scroller));
  
    playlist_global.flatlist = gtk_clist_new_with_titles (2,strTitles);
    gtk_widget_show (playlist_global.flatlist);

  
    gtk_signal_connect_object (GTK_OBJECT (playlist_global.flatlist), "select_row",
			       GTK_SIGNAL_FUNC(playlist_selection_made),
			       NULL);
  
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(playlist_global.flat_scroller), playlist_global.flatlist);

    gtk_drag_dest_set (playlist_global.flatlist,
		       GTK_DEST_DEFAULT_ALL,
		       target_table, n_targets,
		       GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
    gtk_signal_connect( GTK_OBJECT(playlist_global.flatlist), "drag_data_received",
			GTK_SIGNAL_FUNC( flatlist_drag_data_received), NULL);

    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(playlist_global.flat_scroller), 
			TRUE, TRUE, 2);

    /*
     * flatlist-buttons
     */

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 2);
    gtk_widget_show(vbox);

    button = gtk_button_new_with_label ("Up");
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) playlist_move_up,
			       GTK_OBJECT (notebook));
  
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
    gtk_widget_show(button);

    button = gtk_button_new_with_label ("Down");
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) playlist_move_down,
			       GTK_OBJECT (notebook));

    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
    gtk_widget_show(button);

    button = gtk_button_new_with_label ("Add File");
  
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) playlist_add_file,
			       GTK_OBJECT (notebook));
  
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
    gtk_widget_show(button);

    button = gtk_button_new_with_label ("Remove");
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) playlist_remove, 
			       GTK_OBJECT (notebook)); 
  
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
    gtk_widget_show(button);

    button = gtk_button_new_with_label ("Clear");
  
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) playlist_clear, 
			       GTK_OBJECT (notebook)); 
  
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
    gtk_widget_show(button);

    /*
     * one scan button per device:
     */

    pDevList = get_dev_list () ; /* from config_devices */

    pDevCur = g_list_first (pDevList);

    while (pDevCur) {
	gchar  str[80];
	gchar *strDevName;
	strDevName = (gchar *) pDevCur->data ;

/*	fprintf (stderr, "Scan %s\n", strDevName);*/
	sprintf (str, "Scan %s", strDevName);

	button = gtk_button_new_with_label (str);
      
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    (GtkSignalFunc) playlist_scan_device,
			    strDevName);
      
	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
	gtk_widget_show(button);
      
	pDevCur = g_list_next (pDevCur);
    }

    /* add flatlist-page */

    label = gtk_label_new ("Playlist");
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), hbox, label);

    /*
     * the tree ("device view")
     */

    scrolled_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize (scrolled_win, 200, 300);

    label = gtk_label_new ("Devices");
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scrolled_win, label);

    gtk_widget_show (scrolled_win);
  
    playlist_global.tree = (GtkTree *) gtk_tree_new ();
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scrolled_win), 
					   (GtkWidget *) playlist_global.tree);
    gtk_tree_set_selection_mode (playlist_global.tree, GTK_SELECTION_SINGLE);
    gtk_tree_set_view_mode (playlist_global.tree, GTK_TREE_VIEW_LINE);
  
    gtk_widget_show ((GtkWidget *) playlist_global.tree);

    /* One Entry for the files : */
    
    playlist_global.pFileTreeItem = gtk_tree_item_new_with_label ("files");
    
    gtk_tree_append (GTK_TREE(playlist_global.tree), 
		     playlist_global.pFileTreeItem);
    gtk_widget_show (playlist_global.pFileTreeItem);

    playlist_global.pFileTree = gtk_tree_new();
    gtk_tree_item_set_subtree (GTK_TREE_ITEM(playlist_global.pFileTreeItem), 
			       GTK_WIDGET(playlist_global.pFileTree));
    gtk_tree_item_expand (GTK_TREE_ITEM (playlist_global.pFileTreeItem));
  
    /* get data to fill tree */
	   
//    THREADSAFE (scan_devices ());

    /* make tree a drop target */

    gtk_drag_dest_set (GTK_WIDGET(playlist_global.tree),
		       GTK_DEST_DEFAULT_ALL,
		       target_table, n_targets,
		       GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
    gtk_signal_connect( GTK_OBJECT(playlist_global.tree), "drag_data_received",
			GTK_SIGNAL_FUNC( flatlist_drag_data_received), NULL);

    return win;
}



