/*  Open Media System - oms
 *
 * Copyright (C) 1999  Thomas Mirlacher Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "skinstore.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <gdk_imlib.h>

static skin_t *skins[MAX_SKIN];

static gchar *pixmaps[]={"BackgroundImage","Mask"};
static gchar *textnames[]={"Source","Chapter","TNum", "CNum"};
static gchar *statusbuttons[]={"Play","Stop","Pause"};
static gchar *pushbuttons[]={"Next","Prev","Playlist","Config","Subpicture",
			     "Audio","Eject","Close"};
gchar skindir[256];

void init_skinstore() 
{
	int i=0;

	for (i=0; i<MAX_SKIN; i++)
		skins[i] = NULL;
}

skin_t *get_skin (gint nIndex_)
{
	return skins[nIndex_];
}

GdkPixmap *get_skin_pixmap (gint nIndex_) 
{
	skin_t *pSkin = get_skin(nIndex_);

	if (pSkin)
		return pSkin->pixmap;
	else
		return NULL;
}


gboolean testline (gchar *line_, gint nLen_, gchar *itemname_)
{
//	fprintf (stderr, "...testing %s <-> %s (%d)\n", line_, itemname_, nLen_);
	return ((strlen(itemname_)==nLen_) && (strncmp (line_, itemname_, nLen_)==0));
}

gboolean bErr;

GdkPixmap *loadImage (gchar *strName_)
{
	GdkImlibImage *pImage;
	gchar filename[256];
    
	sprintf (filename, "%s/%s",skindir,strName_);
    
	if ((pImage = gdk_imlib_load_image(filename))) {
		GdkPixmap *pix;
		gdk_imlib_render(pImage, pImage->rgb_width, pImage->rgb_height );

		pix = gdk_imlib_move_image(pImage);
		gdk_imlib_destroy_image(pImage);
		return pix;
	} else {
		fprintf (stderr, "Error loading image %s\n", filename);
		return NULL;
	}
}


/*
 * Scanner
 */

gchar **tokens;
gint  nPos;

void scanner_init (gchar *strLine_)
{
	tokens = g_strsplit (strLine_, " ", 0);
	nPos =0;
	bErr = FALSE;
}

gchar *scanner_nextToken ()
{
	gchar *strRes;

	if (bErr)
		return NULL;

	while ((tokens[nPos]) && (!strcmp (tokens[nPos], "")))
		nPos++;

	strRes = tokens[nPos];

	if (strRes==NULL)
		bErr = TRUE;

	nPos++;

	return strRes;
}

int scanner_getInt () 
{
    gchar *strToken = scanner_nextToken ();

    if (bErr)
	return 0;

    return atoi (strToken); 
}

gboolean scanner_getBoolean ()
{
    gchar *strToken = scanner_nextToken ();

    if (bErr)
	return FALSE;

    return (strcmp (strToken, "TRUE")==0) ;
}

void scanner_free ()
{
    g_strfreev (tokens);
}

void parsepixmap (gchar *strLine_, int nID_)
{
    skin_t *pSkin = malloc (sizeof (skin_t));
    gchar *strPixmap;

    scanner_init (strLine_);

//    fprintf (stderr, "PARSE AS PIXMAP : %s\n", strLine_);

    strPixmap = scanner_nextToken();

//    fprintf (stderr, "Loading pixmap : %s...\n", strPixmap);

    pSkin->pixmap = loadImage (strPixmap);

    if (bErr)
    {
	free (pSkin);
	fprintf (stderr,"Error parsing >%s<\n",strLine_);
    }
    else
	skins[10+nID_] = pSkin;

    scanner_free();
}

void parsetext (gchar *strLine_, int nID_)
{
    skin_t *pSkin = malloc (sizeof (skin_t));
    gchar *strPixmap;

    scanner_init (strLine_);

//    fprintf (stderr, "PARSE AS TEXT : %s\n", strLine_);

    strPixmap = scanner_nextToken();

    pSkin->x    = scanner_getInt();
    pSkin->y    = scanner_getInt();
    pSkin->nMax = scanner_getInt();

//    fprintf (stderr, 
//	     "Loading pixmap %s for max %d chars displayed at %d / %d\n",
//	     strPixmap, pSkin->nMax, pSkin->x, pSkin->y);

    pSkin->pixmap = loadImage (strPixmap);

    if (bErr)
    {
	free (pSkin);
	fprintf (stderr,"Error parsing >%s<\n",strLine_);
    }
    else
	skins[20+nID_] = pSkin;

    scanner_free();
}

void parsestatusbutton (gchar *strLine_, int nID_)
{
	skin_t *pSkin = malloc (sizeof (skin_t));
	gchar *strPixmap;

	scanner_init (strLine_);

	strPixmap = scanner_nextToken();

	pSkin->bPrelight = scanner_getBoolean();
	pSkin->bStatus   = scanner_getBoolean();
	pSkin->x         = scanner_getInt();
	pSkin->y         = scanner_getInt();

	pSkin->pixmap = loadImage (strPixmap);

	if (bErr) {
		free (pSkin);
		fprintf (stderr,"Error parsing >%s<\n",strLine_);
	} else
		skins[30+nID_] = pSkin;

	scanner_free();
}

void parsepushbutton (gchar *strLine_, int nID_)
{
	skin_t *pSkin = malloc (sizeof (skin_t));
	gchar *strPixmap;

	scanner_init (strLine_);

	strPixmap = scanner_nextToken();

	pSkin->bPrelight = scanner_getBoolean();
	pSkin->bStatus   = FALSE;
	pSkin->x         = scanner_getInt();
	pSkin->y         = scanner_getInt();

	pSkin->pixmap = loadImage (strPixmap);

	if (bErr) {
		free (pSkin);
		fprintf (stderr,"Error parsing >%s<\n",strLine_);
	} else
		skins[40+nID_] = pSkin;

	scanner_free();
}


void load_skin (gchar *strSkinName_)
{
	FILE   *pFSkin;

	gchar  skinfile[256];

	sprintf (skinfile, "%s/%s/skindata", OMS_SKINDIR,strSkinName_);
	sprintf (skindir,"%s/%s", OMS_SKINDIR,strSkinName_);

	fprintf (stderr, "Opening %s...\n",skinfile);

	pFSkin = fopen (skinfile, "r");

	if (pFSkin != NULL) {
		char line[256];
      
		while (fgets (line, 256, pFSkin)) {
			char *pDPoint;
			line[strlen(line)-1] = 0;

			if (line[0] == '#')	// ignore comments
				continue;

			pDPoint = strchr (line, ' ');

			if (pDPoint != NULL) {
				gint nLength,i;
				nLength = pDPoint-line;

				pDPoint++;

				for (i=0; i<NUM_PIXMAPS; i++)
					if (testline (line, nLength, pixmaps[i]))
						parsepixmap (pDPoint,i);

				for (i=0; i<NUM_TEXTS; i++)
					if (testline (line, nLength, textnames[i]))
						parsetext (pDPoint,i);

				for (i=0; i<NUM_STATUSBUTTONS; i++)
					if (testline (line, nLength, statusbuttons[i]))
						parsestatusbutton (pDPoint,i);

				for (i=0; i<NUM_PUSHBUTTONS; i++)
					if (testline (line, nLength, pushbuttons[i]))
						parsepushbutton (pDPoint,i);
			}

		}

		fclose (pFSkin);
	}

}
