
/*
 * subpic_decode.c - displays DVD subtitles on the screen 
 *
 * Mostly based on hard work by:
 *
 * Copyright (C) 2000   Samuel Hocevar <sam@via.ecp.fr>
 *                       and Michel Lespinasse <walken@via.ecp.fr>
 *
 * Lots of rearranging by:
 *	Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 *	Thomas Mirlacher <dent@cosy.sbg.ac.at>
 *		implemented reassembling
 *		cleaner implementation of SPU are saving
 *		overlaying (proof of concept for now)
 *		... and yes, it works now with oms
 *		added tranparency (provided by the SPU hdr) 
 *		changed structures for easy porting to MGAs DVD mode
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *                                                     
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "spu.h"
#include "overlay.h"
#include "display.h"

#include "tux.h"
#define SPU_BLOCK_SIZE 2048
static u_int *bg_image;
static struct spu_img_struct spu;

static u_int img_width = 720; 
static u_int img_height = 576; 
static u_char inbuf[SPU_BLOCK_SIZE];
static u_int sbuffer_size = 0;
static u_char sbuffer[SPU_BLOCK_SIZE * 5];
static u_int pkt_len;

u_int *create_bg_img (void)
{
	u_char *ptr = (u_char *) bg_img_data;
	u_int y;
	u_int x_start = (img_width - bg_width) / 2;
	u_int y_start = 200;
	u_int *bg_img_ptr = bg_image;

	memset (bg_image, 0, img_width * img_height * sizeof(u_int));

	for (y=y_start; y<bg_height+y_start; y++) {
		bg_img_ptr = bg_image + y*img_width + x_start;

		ptr = bg_img_data[y-y_start];
		while (*ptr != '\0') {
			*bg_img_ptr = bg_palette_to_rgb[(*ptr - bg_start_index)%16];

			bg_img_ptr++;
			ptr++;
		}
	}

	return bg_image;
}


/**
 *
 */

int main (int argc, char *argv[])
{
	int bytes_read;
	FILE *input;

	if (argc != 2) {
		printf("usage: %s file\n", argv[0]);
		exit (1);
	}

	input = fopen (argv[1], "r");

	spuInit ();
	bg_image = (u_int *) malloc (img_width * img_height * sizeof(u_int));

	while ((bytes_read = fread(inbuf, 1, SPU_BLOCK_SIZE, input)) == SPU_BLOCK_SIZE) {
	  decode_spu(inbuf, inbuf + SPU_BLOCK_SIZE);
	}

	sleep (1);
	fclose (input);
	exit (0);
}

u_int buffer_spupack(u_int *length, u_char** start, u_char *end)
{
        u_char *cur = *start;
	u_int ret = 0;
	
	while (sbuffer_size < 2) { // need the first two bytes pack length
	  if (cur >= end)
	    goto done;
	  
	  sbuffer[sbuffer_size++] = *cur++;
	}

	*length = (sbuffer[0] << 8) + sbuffer[1];
	fprintf(stderr, "pack length %d\n", *length);
	
	while (sbuffer_size < *length) {
	  if (cur >= end)
	    goto done;
	  sbuffer[sbuffer_size++] = *cur++;
	}

	// got a whole pack
	sbuffer_size = 0;
	ret = 1;

 done:
	*start = cur;
	return ret;
}

void decode_spu(u_char *data_start, u_char *data_end)
{
	while (buffer_spupack(&pkt_len, &data_start, data_end)) {
	  //DENT: bg_image is the image we get from the mpeg decoder
		bg_image = create_bg_img ();

		if (!spuParseHdr (&spu, sbuffer, pkt_len)) {
			spuParseData (&spu);

//DENT: image is the resulting image from SPU decoding
// (background_image, image_to_overlay, opaque)

			bg_image = overlay (bg_image, &spu, 0.8);
//			spu.x -=50;
//			spu.y -=20;
//			bg_image = overlay (bg_image, &spu, 0.5);

			display_frame (bg_image);

			free (spu.data);
		}

		//uncomment for the real delay
		usleep (spu.time_execute);
		//getchar ();
	}
}
